function spkDat = ClusterSpikes(trodeFile, ops, varargin)	
		
% 11/18/2016    M. Farries

% REQUIRED INPUTS:
%   trodeFile - name of filtered hsd file to be processed (for just 1 tetrode/stereotrode)
%   ops - structure containing spike sorting parameters and options (see ConfigSort script)

% OUTPUTS:
%   spkDat - data structure with fields:
%           .Fs              - sampling rate of data (samples/sec)
%           .upsample
%           .peakLoc         - location of spike peak (within waveform clips)
%           .wavePad
%           .alignWin
%           .vScale          - multiplicative factor that converts raw data to mV (currently assumed to be the same across all channels)
%           .spkDetectSignal
%           .highCut
%           .sincLength
%           .wireSd          - std deviation of raw signal on each wire
%           .W               - matrix specifying channel whitening filter
%           .spkThr          - spike thresholds for each wire in spk detection signal (usually whitened NLE)
%           .waves           - waveforms of extracted spikes
%           .ts              - timestamps (in sample number) of extracted spikes
%           .tsOrig          - timestamps of orignal detected events (before any waveform alignment
%           .chan            - wire/channel number on which spike was detected
%           .id              - cluster id of extracted spikes
%           .ts0

% VARARGs: 
%   preprocessRange - time range for calculating SD and whitening filter (sec)
%   timeRange - time range for spike extraction and sorting (sec)

% REQUIRED M-FILES: ** not complete **
%   DetectSpikes
%   isosplit5
%   getHsdHeader_ver4
%   readHSD
%   snle

% set default values for varargs
preprocessRange = [0 600]; % data time range for computing thresholds and whitening filter (seconds)
timeRange = [0 Inf];       % data time range for template construction (seconds)
gapDur = 60;               % gap, in seconds, separating timestamps of successive data files (if more than one data file is included)
keepWhite = false;         % if true, return spkDat with channel-whitened waveforms
fieldNames = {};           % names of custom fields for spkDat structure 
fieldDat = {};             % data for custom fields

% load varargs
for i = 1 : 2 : length(varargin)
    switch lower(varargin{i})
        case 'preprocessrange',
            preprocessRange = varargin{i+1};
        case 'timerange',
            timeRange = varargin{i+1};
        case 'gapdur'
            gapDur = varargin{i+1};
        case 'keepwhite'
            keepWhite = varargin{i+1};
        case 'fieldname'
            fieldNames = varargin{i+1};
        case 'fielddat'
            fieldDat = varargin{i+1};
        otherwise
            disp('invalid optional argument passed to ClusterSpikes')
            spkDat = null(1);
            return            
    end  
end

if ~iscell(trodeFile)
    trodeFile = {trodeFile};
end

% get the hsd header
hsdHeader = getHsdHeader_ver4(trodeFile{1});
Fs = hsdHeader.sampling_rate;
bytesPerSample = hsdHeader.bytesPerSample;
dataOffset = hsdHeader.dataOffset;
nChan = hsdHeader.num_channels;   
vScale = hsdHeader.vScale;
chanList = 1:nChan;
hsdInfo = dir(trodeFile{1});
datalength = floor((hsdInfo.bytes - dataOffset) / (bytesPerSample * nChan));

% unpack ops elements
verbose = ops.verbose; 
wL = ops.waveLength;
pk = ops.peakLoc;
upsample = ops.upsample; 
batchSize = ops.batchSize;
spkDetectSignal = ops.spkDetectSignal;
relThr = ops.relSpkThr;
noisePeak = ops.noisePeak;
noiseMean = ops.noiseMean;
noiseMaxD = ops.noiseMaxD;
noiseWireCount = ops.noiseWireCount;
spkRange = ops.spkRange;
deadTime = ops.deadTime;
alignWin = ops.alignWin;
maxShift = ops.maxShift;
pcaWin = ops.pcaWin;
numFeatures = ops.numFeatures;
highCut = ops.highCut;
sincLength = ops.sincLength;
batchesPerBlock = ops.batchesPerBlock;
wvFs = Fs * upsample;

% convert ops times from ms to sample numbers
wL = round(wL * Fs * upsample / 1000);
pk = round(pk * Fs * upsample / 1000);
spkRange = round(spkRange * Fs * upsample / 1000);
deadTime = round(deadTime * Fs * upsample / 1000);
alignWin = round(alignWin * Fs * upsample / 1000);
pcaWin = round(pcaWin * Fs * upsample / 1000) + pk;
pcaWin = max(1,pcaWin(1)):min(wL,pcaWin(2));
maxShift = round(maxShift * Fs * upsample / 1000);
ops.alignWin = alignWin;
ops.derivWindow = round(ops.derivWindow * Fs * upsample / 1000);

% put important info into output structure spkDat
spkDat.Fs = Fs;
spkDat.upsample = upsample;
spkDat.peakLoc = pk;
spkDat.wavePad = 0;
spkDat.alignWin = alignWin;
spkDat.vScale = vScale;
spkDat.spkDetectSignal = spkDetectSignal;
spkDat.highCut = highCut;
spkDat.sincLength = sincLength;
for i = 1:length(fieldNames)
    spkDat = setfield(spkDat, fieldNames{i}, fieldDat{i});
end

%% 1) compute signal standard deviation (using median estimator)
timerVal = tic;
if verbose
    fprintf('\nComputing signal standard deviation...');
end

% load data
t1 = max([0 preprocessRange(1)]);
t2 = min([datalength / Fs preprocessRange(2)]);
currentSample = floor(t1 * Fs);
numSamples = round((t2 - t1)*Fs);
dat = readHSD(trodeFile{1}, nChan, dataOffset, Fs, [], 'usesamplelimits', [currentSample, numSamples]);
if upsample > 1
    tmpDat = zeros(size(dat, 1), size(dat, 2) * upsample);
    for w = 1:nChan
        tmpDat(w, :) = sincInterp(dat(w, :), Fs, highCut, upsample*Fs, 'sinclength', sincLength);
    end
    dat = tmpDat;
end

% compute signal standard deviation (median estimator), set thresholds
switch lower(spkDetectSignal)
    case 'snle'
        signal = snle(dat, chanList);
    case 'abs'
        signal = abs(dat);
    otherwise
        signal = dat;
end
spkDat.wireSd = std(dat'); % used for noise rejection post-sort, not spike detection
wireSd = nanmedian(signal, 2) / 0.6745;
spkThr = wireSd * relThr;
if verbose
    fprintf('complete. Elapsed time: %d seconds\n', round(toc(timerVal)));
end
    
%% 2) compute channel-whitening filter, excluding data from spike-like events
if verbose
    fprintf('Computing channel-whitening filter...');
end

% get locations of spike-like events
ts = DetectSpikes(dat, spkThr)';
ts = ts(ts > -spkRange(1));
ts = ts(ts < numSamples - spkRange(2));

% set spiky parts of the data to zero
blankIdx = repmat(ts,1,2) + repmat(spkRange, size(ts));
blankMask = ones(size(dat));
for j = 1:length(blankIdx)
    blankMask(:, blankIdx(j,1):blankIdx(j,2)) = 0;
end
dat = dat .* blankMask;
    
% build covariance matrix, perform SVD to get whitening filter W
CC = (dat * dat')/size(dat,2);
blankCorrector = (blankMask * blankMask')/size(dat,2);
CC = CC ./ blankCorrector;
[E, D] 	= svd(CC);
D = diag(D);
D(D < 1e-8) = 1e-8;
D = 1 ./ D;
W = E * diag(D .^ 0.5) * E';
spkDat.W = W;

if verbose
    fprintf('complete. Elapsed time: %d seconds\n', round(toc(timerVal)));
end

%% 3) recompute thresholds for channel-whitened data
if verbose
    fprintf('Computing whitened signal standard deviation...');
end

dat = readHSD(trodeFile{1}, nChan, dataOffset, Fs, [], 'usesamplelimits', [currentSample, numSamples]);
dat = W * dat;

% compute signal standard deviation (median estimator)
switch lower(spkDetectSignal)
    case 'snle'
        signal = snle(dat, chanList);
    case 'abs'
        signal = abs(dat);
    otherwise
        signal = dat;
end
wireSd = nanmedian(signal, 2) / 0.6745;
spkThr = wireSd * relThr;
spkDat.spkThr = spkThr;
if verbose
    fprintf('complete. Elapsed time: %d seconds\n', round(toc(timerVal)));
end 

%% 4) load and whiten data, extract spikes
waves = [];
tsOrig = [];
chan = [];
peakVal = [];
tsNoise = [];
ts = [];
tsOffset = 0;
spkDat.taskStart = [];
spkDat.taskEnd = [];
gcp; % activate "workers" for parallel processing

% create temporary file to store waveforms as they are extracted
f = fopen([fileparts(trodeFile{1}) filesep 'tmpwave.dat'], 'w+');

if verbose
    fprintf('Whitening data and extracting spikes...\n');
    msg = '';
end

% loop through data files to be sorted together
for k = 1:length(trodeFile)
    
    hsdInfo = dir(trodeFile{k});
    datalength = floor((hsdInfo.bytes - dataOffset) / (bytesPerSample * nChan)); 
    t1 = max([0 timeRange(1)]);
    t2 = min([timeRange(2) datalength / Fs]);
    spkDat.taskStart = [spkDat.taskStart t1 + tsOffset/Fs];
    spkDat.taskEnd = [spkDat.taskEnd t2 + tsOffset/Fs];
    currentSample = floor(t1 * Fs);
    numSamples = round((t2 - t1)*Fs);
    finalSample = currentSample + numSamples - 1;
    overlapSize = 2*wL + 2*ceil(maxShift/2);
    blockSize = batchesPerBlock * (batchSize - overlapSize);
    numBlocks = ceil((numSamples - 2*wL) / blockSize);
    noiseWireCount = min([noiseWireCount nChan]);
    noisePts = (alignWin(1):alignWin(2)) + pk;
    p1 = noisePts(1 : end-upsample);
    p2 = noisePts(1+upsample : end);

    for n = 1:numBlocks
    
        if verbose
            fprintf(repmat('\b', 1, numel(msg)));
            msg = sprintf('file %d of %d, block %d of %d, time elapsed: %d seconds', k, length(trodeFile), n, numBlocks, round(toc(timerVal)));
            fprintf(msg);
        end
    
        % load a block of data, divided into batchesPerBlock batches
        blockSamples = min(finalSample - currentSample, blockSize);
        numBatches = ceil(blockSamples / (batchSize - overlapSize));
        startSample = currentSample + (0 : numBatches-1) * (batchSize-overlapSize);
        dat = cell(1, numBatches);
        for i = 1:numBatches
            dat{i} = readHSD(trodeFile{k}, nChan, dataOffset, Fs, [], 'usesamplelimits', [startSample(i), batchSize]);
            if upsample > 1
                tmpDat = zeros(size(dat{i}, 1), size(dat{i}, 2) * upsample);
                for w = 1:nChan
                    tmpDat(w, :) = sincInterp(dat{i}(w, :), Fs, highCut, wvFs, 'sinclength', sincLength);
                end
                dat{i} = tmpDat;
            end
        end
        
        % extract spikes from each batch in parallel
        spk = cell(1, numBatches);
        orig = cell(1, numBatches);
        wire = cell(1, numBatches);
        ampl = cell(1, numBatches);
        noise = cell(1, numBatches);
        waves = cell(1, numBatches);
        parfor i = 1:numBatches % parfor
            
            % apply channel-whitening filter
            dat{i} = W * dat{i};
            
            % get locations of spike-like events            
            [spk{i}, wire{i}, ampl{i}] = DetectSpikes(dat{i}, spkThr, 'deadTime', deadTime);
            keep = spk{i} > upsample * overlapSize/2 + deadTime & spk{i} <  size(dat{i},2) - upsample * overlapSize/2;
            spk{i} = spk{i}(keep);
            wire{i} = wire{i}(keep);
            ampl{i} = ampl{i}(keep);
            
            % if any spikes were found...
            if ~isempty(spk{i})
            
                % extract spike waveforms for noise detection and waveform alignment
                idx = bsxfun(@plus, spk{i} - pk, (1:wL)');
                wv = reshape(dat{i}(:,idx), nChan, wL, length(spk{i})); % #chan X #pts X #spikes
            
                % flag noise events
                badSpk = [];
                testVal = zeros(nChan, length(spk{i}));
                % check for big peaks
                for w = 1:nChan
                    testVal(w,:) = arrayfun(@(j) max(abs(wv(w,noisePts,j))), 1:length(spk{i}));
                end
                badSpk = union(badSpk, find(sum(testVal > noisePeak, 1) >= noiseWireCount));
                % check for large average value
                for w = 1:nChan
                    testVal(w,:) = arrayfun(@(j) abs(mean(wv(w,noisePts,j))), 1:length(spk{i}));
                end
                badSpk = union(badSpk, find(sum(testVal > noiseMean, 1) >= noiseWireCount));
                % check for large time derivative
                for w = 1:nChan
                    testVal(w,:) = arrayfun(@(j) 1000*max(abs(wv(w,p2,j) - wv(w,p1,j))/Fs), 1:length(spk{i})); 
                end
                badSpk = union(badSpk, find(sum(testVal > noiseMaxD, 1) >= noiseWireCount));
                noise{i} = spk{i}(badSpk); 
                if any(size(noise{i}) == 0)
                    noise{i} = [];
                end
        
                % align waveforms
                orig{i} = round(spk{i} / upsample);
                if ~isempty(alignWin)
                    dt = arrayfun(@(j) AlignWave(wv(:,:,j)', pk, 'alignOps', ops), 1:length(spk{i}));
                    dt(abs(dt) > maxShift) = 0;
                    spk{i} = spk{i} + dt;
                end
        
                % extract full aligned spike waveforms
                idx = bsxfun(@plus, spk{i} - pk, (1:wL)');
                waves{i} = reshape(dat{i}(:,idx), nChan, wL, length(spk{i}));
                spk{i} = round(spk{i} / upsample);
                noise{i} = round(noise{i} / upsample);
            
            end
            
        end
    
        % combine spike info across batches and save waveforms to tmp file
        for i = 1:numBatches
            ts = [ts spk{i} + startSample(i) + tsOffset];
            tsOrig = [tsOrig orig{i} + startSample(i) + tsOffset];
            tsNoise = [tsNoise noise{i} + startSample(i) + tsOffset];
            chan = [chan wire{i}];
            peakVal = [peakVal ampl{i}];
            fwrite(f, reshape(permute(waves{i}, [2 1 3]), nChan*wL, []), 'double');
        end
    
        currentSample = currentSample + blockSize;
        
    end
    
    tsOffset = tsOffset + round((t2 + gapDur) * Fs);
    
end

% load waveforms from tmp file and delete file
clear waves
fseek(f, 0, 'bof');
waves = fread(f, [wL*nChan length(ts)], 'double');
waves = permute(reshape(waves, [wL nChan length(ts)]), [2 1 3]);
fclose(f);
delete([fileparts(trodeFile{1}) filesep 'tmpwave.dat']);

% if no spikes found, abort clustering
if isempty(ts)
    fprintf('\nNo spikes found\n');
    spkDat.waves = zeros(wL, nChan, 0);
    spkDat.ts = [];
    spkDat.tsOrig = [];
    spkDat.chan = [];
    spkDat.peakVal = [];
    spkDat.id = [];
    spkDat.wavePad = 0;
    return
end

%% 5) cluster spikes
if verbose
    fprintf('\nClustering...\n');
end
[~, noiseLoc] = ismember(tsNoise, tsOrig);
spkLoc = setdiff(1:length(tsOrig), noiseLoc);
features = ms_event_features(waves(:,pcaWin,spkLoc), numFeatures);
id = isosplit5(features);
clust = unique(id);
nClust = length(clust);
if verbose
    fprintf('\n%d clusters found. Elapsed time: %d seconds\nFinalizing...', nClust, round(toc(timerVal)));
end

% relabel clusters in descending order of spike count
nSpk = arrayfun(@(i) length(find(id == i)), clust);
[~, iSort] = sort(nSpk, 'descend');
idNew = id;
for i = 1:nClust 
    loc = find(id == iSort(i));
    idNew(loc) = i;
end
id = idNew;

spkDat.waves = permute(waves, [2 1 3]);
spkDat.ts = ts;
spkDat.tsOrig = tsOrig;
spkDat.chan = chan;
spkDat.peakVal = peakVal;
spkDat.id = zeros(1, length(ts));
spkDat.id(spkLoc) = id;

keepWhite = false;
if ~keepWhite 
    spkDat = ReloadWaves(spkDat, trodeFile, 'wavePad', ops.wavePad, 'verbose', false);
end

if verbose
    fprintf('complete. Elapsed time: %d seconds\n\n', round(toc(timerVal)));
end