function FilterHsd(hsdFile, varargin)

% 7/21/2016    M. Farries
% updated 8/23/2018

% usage: FilterData( datFile, varargin )

% This function wavelet filters data and performs common average
% referencing (CAR). The result is saved in .hsd files (one file for each 
% tetrode or stereotrode). Filtering and CAR is applied to all good wires
% regardless of what trodes are specified in "tetrodes" and "refs" (see
% below); those parameters only control what data is saved.

% * Say why!

% REQUIRED INPUTS:
%   datFile - name of the data file (typically .hsd) to be processed, including the path

% VARARGs:
%   tetrodes - vector containing tetrodes to process
%   refs - vector containing refs (stereotrodes) to process
%   startTime -
%   endTime - 
%   maxLevel - integer (see below)
%   saveDir - full path of directory to save processed data in

% REQUIRED M-FILES:
%   channelTypeFromHeader
%   getBytesPerSample
%   getHSDHeader
%   getHSDlength
%   readHSD
%   wavefilter
%   WireList
%   writeHSDheader_ver4

% hard-coded parameters
bytesPerSample = 2;
wname = 'db4';      % specifies Daubechies(4) wavelet filter

% set default values for varargs
maxLevel = 6; % This sets the (high-pass) cutoff frequency for wavelet filtering.
              % cutoff frequency = Fs/(2^(maxLevel+1)) (Fs = sampling rate)
              % For a sampling rate of 31250, maxLevel 6 gives a cutoff frequency of ~244 Hz
              
lowpassSpike = -1;    % cutoff frequency, in Hz, for lowpass filter on spike data (if <0, no lowpass filter)
filtOrderSpike = 5;  % lowpass filter order for spike data 
trodes = [];         % list of "trodes" (typically tetrodes) to be filtered
timeRange   = [0 Inf];
blockDur = 10;
wavelet = true;

[baseDir, hsdFn, ~] = fileparts(hsdFile);
saveDir = [baseDir filesep 'filtered'];
verbose = true;
lfp = true;
lfpDir = [];
filtOrderLfp = 1000; % not used unless decimate uses 'fir' (commented out)
lfpFs = 500;
badChan = [];

% endTime     = getHSDlength('filename', hsdFile);             
% [hsd_path, ~, ~] = fileparts(hsdFile);
% if isempty(hsd_path)
%     hsd_path = pwd;
% end
% pathParts = strsplit(hsd_path, '/');
% saveDir = [hsd_path '/' pathParts{end} '-filtered'];
verbose = true;

for iarg = 1 : 2 : length(varargin)
    
    switch lower(varargin{iarg})
        case 'tetrodes',
            tetrodes    = varargin{iarg + 1};
        case 'refs',
            refs    = varargin{iarg + 1};
        case 'timerange',
            timeRange = varargin{iarg + 1};
        case 'blockdur',
            blockDur = varargin{iarg + 1};
        case 'maxlevel'
            maxLevel = varargin{iarg + 1};
        case 'savedir'
            saveDir = varargin{iarg + 1};
        case 'verbose'
            verbose = varargin{iarg + 1};
        case 'lfp'
            lfp = varargin{iarg+1};
        case 'lfpdir'
            lfpDir = varargin{iarg+1};
        otherwise
            disp('invalid optional argument passed to FilterHsd')
    end
    
end

if ~exist(saveDir, 'dir')
    mkdir(saveDir);
end

if isempty(lfpDir)
    lfpDir = [baseDir filesep 'LFP'];
end

if ~exist(lfpDir, 'dir') & lfp
    mkdir(lfpDir);
end

% get the hsd header
hsdInfo    = dir(hsdFile);
hsdHeader  = getHSDHeader( hsdFile );
Fs         = hsdHeader.main.sampling_rate;
dataOffset = hsdHeader.dataOffset;
numWires   = hsdHeader.main.num_channels;   % total number of wires in the .hsd file

datalength = (hsdInfo.bytes - dataOffset) / (bytesPerSample * numWires);

% create base header for filtered files
headerBase.comment = '';
headerBase.version = 4;
headerBase.sampling_rate = hsdHeader.main.sampling_rate;
headerBase.vScale = 1000/hsdHeader.channel(1).gain;
headerBase.bytesPerSample = bytesPerSample;
headerBase.dataOffset = 20480;

% extract channel names
chanNames = arrayfun(@(x) x.name, hsdHeader.channel, 'UniformOutput', false);
chanNum = arrayfun(@(x) x.original_number, hsdHeader.channel);
badChan = ~logical(arrayfun(@(x) x.good, hsdHeader.channel));

% delete channel names and numbers marked "bad"
chanNames(badChan) = [];
chanNum(badChan) = [];

% separate ECoG and EMG channels from trode channels
egChan = find(cellfun(@(x) strcmp(lower(x(1)), 'e'), chanNames));
nEgChan = length(egChan);
egChanNames = chanNames(egChan);
egChanNum = chanNum(egChan);
trodeChan = setdiff(1:length(chanNames), egChan);
chanNames = chanNames(trodeChan); % remove ECoG/EMG channels from chanNames
chanNum = chanNum(trodeChan); % remove ECoG/EMG channels from chanNum
nTrodeChan = length(trodeChan);

% generate trode list
trodeName = unique(cellfun(@(x) x(1:end-2), chanNames, 'UniformOutput', false));
if ~isempty(trodes)
    trodeName = intersect(trodeName, trodes);
end
trode = [];
nTrodes =  length(trodeName);
for i = 1:nTrodes
    trode(i).name = trodeName{i};
    idx = find(cellfun(@(x) strcmp(x(1:min(length(trode(i).name), length(x))), trode(i).name), chanNames));
    trode(i).wires = chanNum(idx);
end

% create and open files to store wavelet-filtered data (delete if they already exist)
if wavelet
    fTrodeOut = zeros(1, nTrodes);
    for i = 1:nTrodes
        if exist([saveDir filesep trode(i).name '.hsd'], 'file')
            delete([saveDir filesep trode(i).name '.hsd']);
        end
        header = headerBase;
        header.num_channels = length(trode(i).wires);
        idx = find(cellfun(@(x) strcmp(x(1:min(length(trode(i).name), length(x))), trode(i).name), chanNames));
        for j = 1:header.num_channels
            header.channel(j).chip = [];
            header.channel(j).chip_channel = [];
            header.channel(j).impedance = [];
            header.channel(j).name = chanNames{idx(j)};
            header.channel(j).low_cut = Fs/(2^(maxLevel+1));
            header.channel(j).high_cut = hsdHeader.channel(1).high_cut;
        end
        writeHsdHeader_ver4([saveDir filesep trode(i).name '.hsd'], header);
        fTrodeOut(i) = fopen([saveDir filesep trode(i).name '.hsd'], 'a');
    end
end

% create and open files to store wavelet-filtered data (delete if they already exist)
if lfp
    
    r = round(Fs / lfpFs);
    
    % create hsdf files for ECoG/EMG channels
    fEgOut = zeros(1, nEgChan);
    for i = 1:length(fEgOut)
        header = headerBase;
        header.sampling_rate = Fs / r;
        header.num_channels = 1;
        header.channel.chip = [];
        header.channel.chip_channel = [];
        header.channel.impedance = [];
        header.channel.name = egChanNames{i};
        header.channel.low_cut = hsdHeader.channel(1).low_cut;
        header.channel.high_cut = min(Fs / (2*r), hsdHeader.channel(1).high_cut);
        writeHsdHeader_ver4([lfpDir filesep egChanNames{i} '.hsdf'], header);
        fEgOut(i) = fopen([lfpDir filesep egChanNames{i} '.hsdf'], 'a');
    end
    
    % create hsdf files for trode channels
    fLfpOut = zeros(1, nTrodes);
    for i = 1:nTrodes
        header = headerBase;
        header.sampling_rate = Fs / r;
        header.num_channels = length(trode(i).wires);
        idx = find(cellfun(@(x) strcmp(x(1:min(length(trode(i).name), length(x))), trode(i).name), chanNames));
        for j = 1:header.num_channels
            header.channel(j).chip = [];
            header.channel(j).chip_channel = [];
            header.channel(j).impedance = [];
            header.channel(j).name = chanNames{idx(j)};
            header.channel(j).low_cut = max(lowpassSpike, hsdHeader.channel(1).low_cut);
            header.channel(j).high_cut = min(Fs / (2*r), hsdHeader.channel(1).high_cut);
        end
        writeHsdHeader_ver4([lfpDir filesep trode(i).name '.hsdf'], header);
        fLfpOut(i) = fopen([lfpDir filesep trode(i).name '.hsdf'], 'a');
    end
    
end

% construct lowpass filter for trode data
if lowpassSpike > 0
    [b a] = butter(filtOrderSpike, lowpassSpike * 2 / Fs);
end

% set up data loading by block
blockSize   = round(Fs * blockDur);  
overlapSize = round(Fs * 0.1);   % 100 ms overlap between adjacent blocks to avoid edge effects
t1 = max(0, timeRange(1));
t2 = min(datalength / Fs, timeRange(2));
firstSample = round(t1 * Fs);
nSamples = floor((t2 - t1) * Fs);
finalSample = firstSample + nSamples - 1;
numBlocks = ceil(nSamples / blockSize);
gcp;

% process data and write results to files
% datLength = min([datalength (t2 - t1) * Fs]);
msg = '';
if verbose
    fprintf('Filtering data...\n');
end
tic;
for i = 1:numBlocks
    
    if verbose
        fprintf(repmat('\b', 1, length(msg))); % erase previous message with backspaces
        msg = sprintf('block %d of %d, time elapsed: %d seconds', i, numBlocks, round(toc));
        fprintf(msg);
    end
    
    % load data block from hsd file
    currentSample = firstSample + (i-1) * blockSize;
    preSize = min(currentSample, overlapSize);
    postSize = min(finalSample - currentSample - preSize - blockSize + 1, overlapSize);
    blkSize = blockSize;
    if postSize < 0
        postSize = 0;
        blkSize = finalSample - currentSample - preSize + 1;
    end
    numSamples = blockSize + preSize + postSize;
    startSample = currentSample - preSize;
    dat = readHSD(hsdFile, numWires, dataOffset, Fs, [], 'usesamplelimits', [startSample, numSamples], 'bitOrder', 'b');
    
    if lfp
        
        % filter and downsample data
        lfpDat = int16(zeros(size(dat, 1), ceil(size(dat, 2) / r)));
        parfor j = 1:size(dat, 1)
            % lfpDat(j,:) = int16(decimate(double(dat(j,:)), r, filtOrderLfp, 'fir'));
            lfpDat(j,:) = int16(decimate(dat(j,:), r));
        end
        
        % write filtered data block to hsdf file for each ECoG/EMG channel
        preSizeLfp = floor(preSize/r);
        for j = 1:nEgChan
            egDat = lfpDat(egChanNum(j), preSizeLfp + (1 : min(size(lfpDat,2), floor(blkSize/r))));
            fwrite(fEgOut(j), egDat, 'int16');     
        end
    
        % write LFP-filtered data block to hsdf file for each trode
        for j = 1:nTrodes
            trodeDat = lfpDat(trode(j).wires, preSizeLfp + (1 : min(size(lfpDat,2), floor(blkSize/r))));
            fwrite(fLfpOut(j), trodeDat, 'int16');
        end
    
    end
    
    if wavelet
        
        % lowpass filter data
        wvDat = dat(chanNum,:);
        if lowpassSpike > 0
            wvDat = int16(filtfilt(b, a, double(wvDat')))';
        end
        
        % wavelet filter data
        parfor j = 1:nTrodeChan
            [c,l] = wavedec(double(wvDat(j,:)), maxLevel, wname); % get wavelet decomposition
            c = wthcoef('a', c, l);                               % zero out approximation coefficients
            wvDat(j,:) = int16(waverec(c, l, wname));             % reconstruct signal
        end
        
        % apply common average referencing (will actually use median, not mean)
        wvDat = bsxfun(@minus, wvDat, median(wvDat, 1));
    
        % write filtered data block to hsd file for each trode
        if postSize < 0
            postSize = 0;
        end
        for j = 1:length(trode)
            [~, idx] = ismember(trode(j).wires, chanNum);
            trodeDat = wvDat(idx, preSize + 1 : size(wvDat, 2) - postSize);
            fwrite(fTrodeOut(j), trodeDat, 'int16');
        end   
        
    end 
    
end

if wavelet
    arrayfun(@(f) fclose(f), fTrodeOut);    % close trode hsd files
end
if lfp
    arrayfun(@(f) fclose(f), [fEgOut fLfpOut]);    % close hsdf (LFP) files
end


if verbose
    fprintf('\nFiltering complete. Total time elapsed: %d seconds\n', round(toc));
end
