 function FilterIntan(datDir, varargin)

% 3/24/2017    M. Farries

% usage: FilterIntan( datDir, varargin )

% This function wavelet filters data and performs common average
% referencing (CAR). The result is saved in .hsd files (one file for each 
% tetrode or stereotrode). Filtering and CAR is applied to all good wires
% regardless of what trodes are specified in "tetrodes" and "refs" (see
% below); those parameters only control what data is saved.

% * Say why!

% REQUIRED INPUTS:
%   datFile - name of the data file (typically .hsd) to be processed, including the path

% VARARGs:
%   trodes - vector containing tetrodes to process

%   maxLevel - integer (see below)
%   saveDir - full path of directory to save processed data in

% REQUIRED M-FILES:
%   ReadRhdInfo
%   writeHsdHeader_ver4

% hard-coded parameters
bytesPerSample = 2;
wname = 'db4';      % specifies Daubechies(4) wavelet filter

% set default values for varargs
maxLevel = 6; % This sets the (high-pass) cutoff frequency for wavelet filtering.
              % cutoff frequency = Fs/(2^(maxLevel+1)) (Fs = sampling rate)
              % For a sampling rate of 30000/sec, maxLevel 6 gives a cutoff frequency of ~234 Hz
              
lowpassSpike = -1;    % cutoff frequency, in Hz, for lowpass filter on spike data (if <0, no lowpass filter)
filtOrderSpike = 5;  % lowpass filter order for spike data 
trodes = [];         % list of "trodes" (typically tetrodes) to be filtered
timeRange = [0 Inf]; % time range of recording (in seconds) to filter
blockDur = 10;       % size of data chunks (in sec) to be processed at a time
polarity = -1;       % flip the spikes upside down, if that's the way you like it
wavelet = true;      % for LFP only, make lfp = true and wavelet = false

pathParts = strsplit(datDir, filesep);
pathParts{end} = [pathParts{end} '-filtered'];
saveDir = strjoin(pathParts, filesep);
verbose = true;
lfp = false;
lfpDir = [];
filtOrderLfp = 1000; % not used unless decimate uses 'fir' (commented out)
lfpFs = 500;
badChan = [];

for i = 1 : 2 : length(varargin)
    
    switch lower(varargin{i})
        case 'maxlevel'
            maxLevel = varargin{i+1};
        case 'lowcut'
            lowpassSpike = varargin{i+1};
        case 'filtorderspike'
            filtOrderSpike = varargin{i+1};
        case 'trodes',
            trodes = varargin{i+1};
        case 'timerange',
            timeRange = varargin{i+1};
        case 'blockdur',
            blockDur = varargin{i+1};
        case 'polarity',
            polarity = varargin{i+1};
        case 'savedir'
            saveDir = varargin{i+1};
        case 'verbose'
            verbose = varargin{i+1};
        case 'lfp'
            lfp = varargin{i+1};
        case 'filtorderlfp'
            filtOrderLfp = varargin{i+1};
        case 'lfpfs'
            lfpFs = varargin{i+1};
        case 'lfpdir'
            lfpDir = varargin{i+1};
        case 'badchan'
            badChan = varargin{i+1};
        case 'wavelet'
            wavelet = varargin{i+1};
        otherwise
            disp('invalid optional argument passed to FilterIntan')
    end
    
end

if ~exist(saveDir, 'dir') & wavelet
    mkdir(saveDir);
end

if isempty(lfpDir)
    lfpDir = saveDir;
end

if ~exist(lfpDir, 'dir') & lfp
    mkdir(lfpDir);
end

% load RHD info
rhdFile = [datDir filesep 'info.rhd'];
rhdInfo = ReadRhdInfo(rhdFile);
Fs = rhdInfo.Fs;

% correct mismatch in data files (e.g., files in rhdInfo that are unavailable)
rhdList = arrayfun(@(x) x.native_name, rhdInfo.amp, 'UniformOutput', false);
dirList = dir([datDir filesep 'amp-*.dat']);
dirList = arrayfun(@(x) x.name(5:9), dirList, 'UniformOutput', false)';
[~, del] = setdiff(rhdList, dirList);
rhdInfo.amp(del) = [];

% create base hsdHeader
headerBase.comment = '';
headerBase.version = 4;
headerBase.sampling_rate = Fs;
headerBase.vScale = 0.000195;
headerBase.bytesPerSample = bytesPerSample;
headerBase.dataOffset = 20480;

% extract "custom name" recorded channels
chanNames = arrayfun(@(x) x.custom_name, rhdInfo.amp, 'UniformOutput', false);

% delete channels marked "bad"
chanNames(badChan) = [];
rhdInfo.amp(badChan) = [];

% separate ECoG and EMG channels from trode channels
ecogChan = find(cellfun(@(x) strcmp(lower(x(1:3)), 'eco'), chanNames));
nEcogChan = length(ecogChan);
emgChan = find(cellfun(@(x) strcmp(lower(x(1:3)), 'emg'), chanNames));
nEmgChan = length(emgChan);
trodeChan = setdiff(1:length(chanNames), [ecogChan emgChan]);
chanNames = chanNames(trodeChan); % remove ECoG/EMG channels from chanNames
ecogChan = rhdInfo.amp(ecogChan);
emgChan = rhdInfo.amp(emgChan);
trodeChan = rhdInfo.amp(trodeChan);
nTrodeChan = length(trodeChan);

% generate trode list
trodeName = unique(cellfun(@(x) x(1:end-2), chanNames, 'UniformOutput', false));
if ~isempty(trodes)
    trodeName = intersect(trodeName, trodes);
end
trode = [];
nTrodes =  length(trodeName);
for i = 1:nTrodes
    trode(i).name = trodeName{i};
    trode(i).wires = find(cellfun(@(x) strcmp(x(1:min(length(trode(i).name), length(x))), trode(i).name), chanNames));
end

% create and open files to store filtered data (delete if they already exist)
if wavelet
    fTrodeOut = zeros(1, nTrodes);
    for i = 1:nTrodes
        if exist([saveDir filesep trode(i).name '.hsd'], 'file')
            delete([saveDir filesep trode(i).name '.hsd']);
        end
        header = headerBase;
        header.num_channels = length(trode(i).wires);
        for j = 1:header.num_channels
            header.channel(j).chip = trodeChan(trode(i).wires(j)).board_stream;
            header.channel(j).chip_channel = trodeChan(trode(i).wires(j)).chip_channel;
            header.channel(j).impedance = trodeChan(trode(i).wires(j)).impedance;
            header.channel(j).name = lower(trodeChan(trode(i).wires(j)).custom_name);
            header.channel(j).low_cut = max(Fs/(2^(maxLevel+1)), rhdInfo.bandwidth(1));
            header.channel(j).high_cut = min(lowpassSpike, rhdInfo.bandwidth(2));
        end
        writeHsdHeader_ver4([saveDir filesep trode(i).name '.hsd'], header);
        fTrodeOut(i) = fopen([saveDir filesep trode(i).name '.hsd'], 'a');
    end
end

% set up data loading by block
datInfo = dir([datDir filesep 'amp-' trodeChan(1).native_name '.dat']);
datalength = datInfo.bytes / bytesPerSample;
blockSize   = round(Fs * blockDur);  
overlapSize = round(Fs * 0.1);   % 100 ms overlap between adjacent blocks to avoid edge effects
t1 = max(0, timeRange(1));
t2 = min(datalength / Fs, timeRange(2));
firstSample = round(t1 * Fs);
nSamples = floor((t2 - t1) * Fs);
finalSample = firstSample + nSamples - 1;
numBlocks = ceil(nSamples / blockSize);
gcp;

% set up LFP extraction, perform LFP processing on ECoG/EMG channels
tic;
if lfp
    
    r = round(Fs / lfpFs);
    
    if verbose
        fprintf('Filtering and downsampling ECoG/EMG data...\n');
    end
    msg = '';
    
    % open Intan ECoG/EMG data files for reading
    eChan = [ecogChan emgChan];
    fEgIn = arrayfun(@(x) fopen([datDir filesep 'amp-' x.native_name '.dat']), eChan);
    
    % create hsdf files for ECoG/EMG channels
    fEgOut = zeros(1, nEcogChan + nEmgChan);
    for i = 1:length(fEgOut)
        header = headerBase;
        header.sampling_rate = Fs / r;
        header.num_channels = 1;
        header.channel.chip = eChan(i).board_stream;
        header.channel.chip_channel = eChan(i).chip_channel;
        header.channel.impedance = eChan(i).impedance;
        header.channel.name = lower(eChan(i).custom_name);
        header.channel.low_cut = rhdInfo.bandwidth(1);
        header.channel.high_cut = min(Fs / (2*r), rhdInfo.bandwidth(2));
        writeHsdHeader_ver4([lfpDir filesep eChan(i).custom_name '.hsdf'], header);
        fEgOut(i) = fopen([lfpDir filesep eChan(i).custom_name '.hsdf'], 'a');
    end
    
    for i = 1:numBlocks
        
        if verbose
            fprintf(repmat('\b', 1, length(msg))); % erase previous message with backspaces
            msg = sprintf('block %d of %d, time elapsed: %d seconds', i, numBlocks, round(toc));
            fprintf(msg);
        end
        
        % load data from Intan files
        currentSample = firstSample + (i-1) * blockSize;
        preSize = min(currentSample, overlapSize);
        postSize = min(finalSample - currentSample - preSize - blockSize + 1, overlapSize);
        blkSize = blockSize;
        if postSize < 0
            postSize = 0;
            blkSize = finalSample - currentSample - preSize + 1;
        end
        dat = zeros(length(eChan), preSize+blkSize+postSize);
        for j = 1:length(eChan)
            fseek(fEgIn(j), (currentSample - preSize) * bytesPerSample, 'bof');
            dat(j,:) = polarity * double(fread(fEgIn(j), preSize+blkSize+postSize, 'int16'));
        end
        
        % filter and downsample data
        lfpDat = int16(zeros(length(eChan), ceil(size(dat, 2) / r)));
        parfor j = 1:length(eChan)
            % lfpDat(j,:) = int16(decimate(double(dat(j,:)), r, filtOrderLfp, 'fir'));
            lfpDat(j,:) = int16(decimate(dat(j,:), r));
        end
        
        % write filtered data block to hsdf file for each ECoG/EMG channel
        preSize = floor(preSize/r);
        for j = 1:length(eChan)
            fwrite(fEgOut(j), lfpDat(j, preSize + (1 : min(size(lfpDat,2), floor(blkSize/r)))), 'int16');     
        end 
    
    end
    fprintf('\n');
    
    % close ECoG/EMG files
    arrayfun(@(f) fclose(f), fEgIn);     % close Intan files
    arrayfun(@(f) fclose(f), fEgOut);    % close hsdf files
    
    % create hsdf files for trode channels
    fLfpOut = zeros(1, nTrodes);
    for i = 1:nTrodes
        header = headerBase;
        header.sampling_rate = Fs / r;
        header.num_channels = length(trode(i).wires);
        for j = 1:header.num_channels
            header.channel(j).chip = trodeChan(trode(i).wires(j)).board_stream;
            header.channel(j).chip_channel = trodeChan(trode(i).wires(j)).chip_channel;
            header.channel(j).impedance = trodeChan(trode(i).wires(j)).impedance;
            header.channel(j).name = lower(trodeChan(trode(i).wires(j)).custom_name);
            header.channel(j).low_cut = rhdInfo.bandwidth(1);
            header.channel(j).high_cut = min(Fs / (2*r), rhdInfo.bandwidth(2));
        end
        writeHsdHeader_ver4([lfpDir filesep trode(i).name '.hsdf'], header);
        fLfpOut(i) = fopen([lfpDir filesep trode(i).name '.hsdf'], 'a');
    end
    
end

% construct lowpass filter for trode data
if lowpassSpike > 0
    [b a] = butter(filtOrderSpike, lowpassSpike * 2 / Fs);
end

% open Intan trode data files for reading
fTrodeIn = arrayfun(@(x) fopen([datDir filesep 'amp-' x.native_name '.dat']), trodeChan);

% filter data and write results to hsd/hsdf files
msg = '';
if verbose
    fprintf('Filtering spike data...\n');
end

for i = 1:numBlocks
    
    if verbose
        fprintf(repmat('\b', 1, length(msg))); % erase previous message with backspaces
        msg = sprintf('block %d of %d, time elapsed: %d seconds', i, numBlocks, round(toc));
        fprintf(msg);
    end
    
    % load data from Intan files
    currentSample = firstSample + (i-1) * blockSize;
    preSize = min(currentSample, overlapSize);
    postSize = min(finalSample - currentSample - preSize - blockSize + 1, overlapSize);
    blkSize = blockSize;
    if postSize < 0
        postSize = 0;
        blkSize = finalSample - currentSample - preSize + 1;
    end
    dat = zeros(nTrodeChan, preSize+blkSize+postSize);
    for j = 1:nTrodeChan
        fseek(fTrodeIn(j), (currentSample - preSize) * bytesPerSample, 'bof');
        dat(j,:) = polarity * double(fread(fTrodeIn(j), preSize+blkSize+postSize, 'int16'));
    end
    
    % LFP extraction 
    if lfp
        % filter and downsample data
        lfpDat = int16(zeros(nTrodeChan, ceil((preSize+blkSize+postSize) / r)));
        parfor j = 1:nTrodeChan
            % lfpDat(j,:) = int16(decimate(double(dat(j,:)), r, filtOrderLfp, 'fir'));
            lfpDat(j,:) = int16(decimate(dat(j,:), r));
        end
        
        % write filtered data block to hsdf file for each trode
        preLfp = floor(preSize/r);
        for j = 1:length(trode)
            fwrite(fLfpOut(j), lfpDat(trode(j).wires, preLfp + (1 : min(size(lfpDat,2), floor(blkSize/r)))), 'int16');
        end
    end
    
    if wavelet
        
        % lowpass filter data
        if lowpassSpike > 0
            dat = int16(filtfilt(b, a, double(dat')))';
        end
    
        % wavelet filter data
        parfor j = 1:nTrodeChan
            [c,l] = wavedec(double(dat(j,:)), maxLevel, wname); % get wavelet decomposition
            c = wthcoef('a', c, l);                             % zero out approximation coefficients
            dat(j,:) = int16(waverec(c, l, wname));             % reconstruct signal
        end
    
        % apply common average referencing (will actually use median, not mean)
        dat = bsxfun(@minus, dat, median(dat, 1));
    
        % write filtered data block to hsd file for each trode
        if postSize < 0
            postSize = 0;
        end
        for j = 1:length(trode)
            fwrite(fTrodeOut(j), dat(trode(j).wires, preSize + 1 : size(dat, 2) - postSize), 'int16');
        end    
    
    end
    
end

arrayfun(@(f) fclose(f), fTrodeIn);     % close trode Intan files
if wavelet
    arrayfun(@(f) fclose(f), fTrodeOut);    % close trode hsd files
end
if lfp
    arrayfun(@(f) fclose(f), fLfpOut);    % close trode hsdf (LFP) files
end

if verbose
    fprintf('\nFiltering complete. Total time elapsed: %d seconds\n', round(toc));
end
