function FilterIntanSiProbe(datDir, varargin)

% 8/28/2018    M. Farries
% currently works on single-channel-per-file-format only

% usage: FilterIntan( datDir, varargin )

% This function wavelet filters data and performs common average
% referencing (CAR). The result is saved in a single .hsd file. 
% It also creates a channel map file to be used by Kilosort.


% REQUIRED INPUTS:
%   datDir - directory containing raw Intan data, including full path

% VARARGs:

%   maxLevel - integer (see below)
%   saveDir - full path of directory to save processed data in

% REQUIRED M-FILES:
%   ReadRhdInfo
%   writeHsdHeader_ver4

% hard-coded parameters
bytesPerSample = 2;
wname = 'db4';      % specifies Daubechies(4) wavelet filter

% set default values for varargs
maxLevel = 6; % This sets the (high-pass) cutoff frequency for wavelet filtering.
              % cutoff frequency = Fs/(2^(maxLevel+1)) (Fs = sampling rate)
              % For a sampling rate of 30000/sec, maxLevel 6 gives a cutoff frequency of ~234 Hz
              
lowpassSpike = -1;    % cutoff frequency, in Hz, for lowpass filter on spike data (if <0, no lowpass filter)
filtOrderSpike = 5;  % lowpass filter order for spike data 
timeRange = [0 Inf]; % time range of recording (in seconds) to filter
blockDur = 60;       % size of data chunks (in sec) to be processed at a time
polarity = 1;       % flip the spikes upside down, if that's the way you like it
wavelet = true;      % for LFP only, make lfp = true and wavelet = false

ports = 'CD'; % for channel mapping; 1st letter is port for probe front, 2nd for probe back

pathParts = strsplit(datDir, filesep);
saveDir = strjoin(pathParts(1:end-1), filesep);
waveName = [saveDir filesep pathParts{end} '-filtered.hsd']; % filename to save wavelet-filtered data
verbose = true;
lfp = false;
lfpName = [saveDir filesep pathParts{end} '-LFP.hsd']; % filename to save lowpass-filtered (LFP) data
filtOrderLfp = 1000; % not used unless decimate uses 'fir' (commented out)
lfpFs = 500;

for i = 1 : 2 : length(varargin)
    
    switch lower(varargin{i})
        case 'maxlevel'
            maxLevel = varargin{i+1};
        case 'lowcut'
            lowpassSpike = varargin{i+1};
        case 'filtorderspike'
            filtOrderSpike = varargin{i+1};
        case 'trodes',
            trodes = varargin{i+1};
        case 'timerange',
            timeRange = varargin{i+1};
        case 'blockdur',
            blockDur = varargin{i+1};
        case 'polarity',
            polarity = varargin{i+1};
        case 'savename'
            waveName = varargin{i+1};
        case 'wavename'
            waveName = varargin{i+1};
        case 'verbose'
            verbose = varargin{i+1};
        case 'lfp'
            lfp = varargin{i+1};
        case 'filtorderlfp'
            filtOrderLfp = varargin{i+1};
        case 'lfpfs'
            lfpFs = varargin{i+1};
        case 'lfpname'
            lfpName = varargin{i+1};
        case 'wavelet'
            wavelet = varargin{i+1};
        case 'ports'
            ports = upper(varargin{i+1});
        otherwise
            disp('invalid optional argument passed to FilterIntan')
    end
    
end

% load RHD info
rhdFile = [datDir filesep 'info.rhd'];
rhdInfo = ReadRhdInfo(rhdFile);
Fs = rhdInfo.Fs;

% create base hsdHeader
headerBase.comment = '';
headerBase.version = 4;
headerBase.sampling_rate = Fs;
headerBase.vScale = 0.000195;
headerBase.bytesPerSample = bytesPerSample;
headerBase.dataOffset = 20480;

% extract port and channel number of recorded channels
port = arrayfun(@(x) x.port_prefix, rhdInfo.amp);
chanNames = arrayfun(@(x) x.native_name, rhdInfo.amp, 'UniformOutput', false);
chanNum = cellfun(@(x) strsplit(x, '-'), chanNames, 'UniformOutput', false);
chanNum = cellfun(@(x) str2num(x{2}), chanNum);

% separate ECoG and EMG channels from trode channels
egNum = {'-018', '-019'};
egNames = arrayfun(@(x) [x egNum{1}], unique(port), 'UniformOutput', false);
egNames = [egNames arrayfun(@(x) [x egNum{2}], unique(port), 'UniformOutput', false)];
egIdx = cellfun(@(x) find(cellfun(@(y) strcmp(x,y), chanNames)), egNames);
impedance = arrayfun(@(x) x.impedance, rhdInfo.amp(egIdx));
egIdx = egIdx(impedance < 100000);
nEgChan = length(egIdx);
trodeIdx = setdiff(1:length(chanNames), egIdx);
chanNum = chanNum(trodeIdx); % remove ECoG/EMG channels from chanNum
egChan = rhdInfo.amp(egIdx);
trodeChan = rhdInfo.amp(trodeIdx);
nTrodeChan = length(trodeChan);

% save channel map (for KiloSort)
port = arrayfun(@(x) find(x == ports), port(trodeIdx));
[xcoords ycoords] = ChanMapDan256(chanNum, port);
connected = true(nTrodeChan, 1);
chanMap   = 1:nTrodeChan;
chanMap0ind = chanMap - 1;
kcoords   = ones(nTrodeChan, 1);
fs = Fs;
save([saveDir filesep 'chanMap.mat'], ...
    'chanMap','connected', 'xcoords', 'ycoords', 'kcoords', 'chanMap0ind', 'fs');

% create and open file to store filtered data (delete if it already exist)
if wavelet
    if exist(waveName, 'file')
        delete(waveName);
    end
    header = headerBase;
    header.num_channels = nTrodeChan;
    for j = 1:header.num_channels
        header.channel(j).chip = trodeChan(j).board_stream;
        header.channel(j).chip_channel = trodeChan(j).chip_channel;
        header.channel(j).impedance = trodeChan(j).impedance;
        header.channel(j).name = lower(trodeChan(j).custom_name);
        header.channel(j).low_cut = max(Fs/(2^(maxLevel+1)), rhdInfo.bandwidth(1));
        header.channel(j).high_cut = min(lowpassSpike, rhdInfo.bandwidth(2));
    end
    writeHsdHeader_ver4(waveName, header);
    fOut = fopen(waveName, 'a');
end

% set up data loading by block
datInfo = dir([datDir filesep 'amp-' trodeChan(1).native_name '.dat']);
datalength = datInfo.bytes / bytesPerSample;
blockSize   = round(Fs * blockDur);  
overlapSize = round(Fs * 0.1);   % 100 ms overlap between adjacent blocks to avoid edge effects
t1 = max(0, timeRange(1));
t2 = min(datalength / Fs, timeRange(2));
firstSample = round(t1 * Fs);
nSamples = floor((t2 - t1) * Fs);
finalSample = firstSample + nSamples - 1;
numBlocks = ceil(nSamples / blockSize);
gcp;

% set up LFP extraction, perform LFP processing on ECoG/EMG channels *** NOT DONE ***
tic;
if lfp
    
    r = round(Fs / lfpFs);
    
    if verbose
        fprintf('Filtering and downsampling ECoG/EMG data...\n');
    end
    msg = '';
    
    % open Intan ECoG/EMG data files for reading
    fEgIn = arrayfun(@(x) fopen([datDir filesep 'amp-' x.native_name '.dat']), egChan);
    
    % create hsdf files for ECoG/EMG channels
    fEgOut = zeros(1, nEgChan);
    for i = 1:length(fEgOut)
        header = headerBase;
        header.sampling_rate = Fs / r;
        header.num_channels = 1;
        header.channel.chip = egChan(i).board_stream;
        header.channel.chip_channel = egChan(i).chip_channel;
        header.channel.impedance = egChan(i).impedance;
        header.channel.name = lower(egChan(i).custom_name);
        header.channel.low_cut = rhdInfo.bandwidth(1);
        header.channel.high_cut = min(Fs / (2*r), rhdInfo.bandwidth(2));
        writeHsdHeader_ver4([saveDir filesep 'ECoG' num2str(i) '.hsdf'], header);
        fEgOut(i) = fopen([saveDir filesep 'ECoG' num2str(i) '.hsdf'], 'a');
    end
    
    for i = 1:numBlocks
        
        if verbose
            fprintf(repmat('\b', 1, length(msg))); % erase previous message with backspaces
            msg = sprintf('block %d of %d, time elapsed: %d seconds', i, numBlocks, round(toc));
            fprintf(msg);
        end
        
        % load data from Intan files
        currentSample = firstSample + (i-1) * blockSize;
        preSize = min(currentSample, overlapSize);
        postSize = min(finalSample - currentSample - preSize - blockSize + 1, overlapSize);
        blkSize = blockSize;
        if postSize < 0
            postSize = 0;
            blkSize = finalSample - currentSample - preSize + 1;
        end
        dat = zeros(length(egChan), preSize+blkSize+postSize);
        for j = 1:length(egChan)
            fseek(fEgIn(j), (currentSample - preSize) * bytesPerSample, 'bof');
            dat(j,:) = polarity * double(fread(fEgIn(j), preSize+blkSize+postSize, 'int16'));
        end
        
        % filter and downsample data
        lfpDat = int16(zeros(length(egChan), ceil(size(dat, 2) / r)));
        parfor j = 1:length(egChan)
            % lfpDat(j,:) = int16(decimate(double(dat(j,:)), r, filtOrderLfp, 'fir'));
            lfpDat(j,:) = int16(decimate(dat(j,:), r));
        end
        
        % write filtered data block to hsdf file for each ECoG/EMG channel
        preSize = floor(preSize/r);
        for j = 1:length(egChan)
            fwrite(fEgOut(j), lfpDat(j, preSize + (1 : min(size(lfpDat,2), floor(blkSize/r)))), 'int16');     
        end 
    
    end
    fprintf('\n');
    
    % close ECoG/EMG files
    arrayfun(@(f) fclose(f), fEgIn);     % close Intan files
    arrayfun(@(f) fclose(f), fEgOut);    % close hsdf files
    
    % create hsdf files for trode channels
%     fLfpOut = zeros(1, nTrodes);
%     for i = 1:nTrodes
%         header = headerBase;
%         header.sampling_rate = Fs / r;
%         header.num_channels = length(trode(i).wires);
%         for j = 1:header.num_channels
%             header.channel(j).chip = trodeChan(j).board_stream;
%             header.channel(j).chip_channel = trodeChan(j).chip_channel;
%             header.channel(j).impedance = trodeChan(j).impedance;
%             header.channel(j).name = lower(trodeChan(j).custom_name);
%             header.channel(j).low_cut = rhdInfo.bandwidth(1);
%             header.channel(j).high_cut = min(Fs / (2*r), rhdInfo.bandwidth(2));
%         end
%         writeHsdHeader_ver4([lfpName filesep trode(i).name '.hsdf'], header);
%         fLfpOut(i) = fopen([lfpName filesep trode(i).name '.hsdf'], 'a');
%     end
    
end

% ******** FIX AND REMOVE *************
lfp = false;
numBlocks = 10;
% *************************************

% construct lowpass filter for trode data
if lowpassSpike > 0
    [b a] = butter(filtOrderSpike, lowpassSpike * 2 / Fs);
end

% open Intan trode data files for reading
fChanIn = arrayfun(@(x) fopen([datDir filesep 'amp-' x.native_name '.dat']), trodeChan);

% filter data and write results to hsd/hsdf files
msg = '';
if verbose
    fprintf('Filtering spike data...\n');
end

for i = 1:numBlocks
    
    if verbose
        fprintf(repmat('\b', 1, length(msg))); % erase previous message with backspaces
        msg = sprintf('block %d of %d, time elapsed: %d seconds', i, numBlocks, round(toc));
        fprintf(msg);
    end
    
    % load data from Intan files
    currentSample = firstSample + (i-1) * blockSize;
    preSize = min(currentSample, overlapSize);
    postSize = min(finalSample - currentSample - preSize - blockSize + 1, overlapSize);
    blkSize = blockSize;
    if postSize < 0
        postSize = 0;
        blkSize = finalSample - currentSample - preSize + 1;
    end
    dat = zeros(nTrodeChan, preSize+blkSize+postSize);
    for j = 1:nTrodeChan
        fseek(fChanIn(j), (currentSample - preSize) * bytesPerSample, 'bof');
        dat(j,:) = polarity * double(fread(fChanIn(j), preSize+blkSize+postSize, 'int16'));
    end
    
    % LFP extraction 
    if lfp
        % filter and downsample data
        lfpDat = int16(zeros(nTrodeChan, ceil((preSize+blkSize+postSize) / r)));
        parfor j = 1:nTrodeChan
            % lfpDat(j,:) = int16(decimate(double(dat(j,:)), r, filtOrderLfp, 'fir'));
            lfpDat(j,:) = int16(decimate(dat(j,:), r));
        end
        
        % write filtered data block to hsdf file for each trode
        preLfp = floor(preSize/r);
        for j = 1:length(trode)
            fwrite(fLfpOut(j), lfpDat(trode(j).wires, preLfp + (1 : min(size(lfpDat,2), floor(blkSize/r)))), 'int16');
        end
    end
    
    if wavelet
        
        % lowpass filter data
        if lowpassSpike > 0
            dat = int16(filtfilt(b, a, double(dat')))';
        end
    
        % wavelet filter data
        parfor j = 1:nTrodeChan
            [c,l] = wavedec(double(dat(j,:)), maxLevel, wname); % get wavelet decomposition
            c = wthcoef('a', c, l);                             % zero out approximation coefficients
            dat(j,:) = int16(waverec(c, l, wname));             % reconstruct signal
        end
    
        % apply common average referencing (will actually use median, not mean)
        dat = bsxfun(@minus, dat, median(dat, 1));
    
        % write filtered data block to hsd file for each trode
        if postSize < 0
            postSize = 0;
        end
        fwrite(fOut, dat(:, preSize + 1 : size(dat, 2) - postSize), 'int16');  
    
    end
    
end

arrayfun(@(f) fclose(f), fChanIn);     % close trode Intan files
if wavelet
    arrayfun(@(f) fclose(f), fOut);    % close trode hsd files
end
if lfp
    arrayfun(@(f) fclose(f), fLfpOut);    % close trode hsdf (LFP) files
end

if verbose
    fprintf('\nFiltering complete. Total time elapsed: %d seconds\n', round(toc));
end
