function data = PlotIntanData(datDir, varargin)

% hard-coded parameters
vScale = 0.000195;
bytesPerSample = 2;

% default values of optional arguments
trodeList = -1;
chanList = [];
timeRange = [0 20];
yRange = [-2 2];
returnData = false;
makePlot = true;

for i = 1 : 2 : length(varargin) 
    switch lower(varargin{i})
        case 'returndata'
            returnData = varargin{i+1};
        case 'makeplot'
            makePlot = varargin{i+1};
        case 'trode'
            trodeList = varargin{i+1};
        case 'chan'
            chanList = varargin{i+1};
        case 'timerange'
            timeRange = varargin{i+1};
        case 'yrange'
            yRange = varargin{i+1};
        otherwise
            disp('Invalid optional argument passed to PlotIntanData')
    end
end


% load RHD info
rhdFile = [datDir filesep 'info.rhd'];
rhdInfo = ReadRhdInfo(rhdFile);
Fs = rhdInfo.Fs;

% extract "custom name" recorded channels
if isstr(trodeList)
    trodeList = {trodeList};
end
chanNumbers = arrayfun(@(x) str2num(x.native_name(3:end)), rhdInfo.amp);
chanNames = arrayfun(@(x) x.custom_name, rhdInfo.amp, 'UniformOutput', false);
trodeNames = unique(cellfun(@(x) x(1:end-2), chanNames, 'UniformOutput', false));

% assemble list of channels to load/plot
if isempty(chanList)    
    % use trode names
    if ~iscell(trodeList) && trodeList ==  -1
        trodeList = trodeNames;
    end    
    ampIdx = cell(1, length(trodeList));
    for i = 1:length(trodeList)
        ampIdx{i} = find(cellfun(@(x) strcmp(x(1:min(length(trodeList{i}), length(x))), trodeList{i}), chanNames));
        wireNum = arrayfun(@(i) str2num(chanNames{i}(end)), ampIdx{i});
        [~, iSort] = sort(wireNum);
        ampIdx{i} = ampIdx{i}(iSort);
    end    
else    
    % use channel numbers
    [~, ampIdx] = ismember(chanList, chanNumbers);
    ampIdx = {ampIdx};   
end

% load data and/or make plots
firstSample = round(timeRange(1) * Fs);
nSamples = floor(diff(timeRange) * Fs);
times = timeRange(1) + (0:nSamples-1)/Fs;
data.t = times;
data.v = [];
data.chanNames = {};
for i = length(ampIdx):-1:1
    
    % open Intan data files for reading
    fIn = arrayfun(@(j) fopen([datDir filesep 'amp-' rhdInfo.amp(j).native_name '.dat']), ampIdx{i});
    
    % load data
    dat = zeros(length(ampIdx{i}), nSamples);
    for j = 1:length(ampIdx{i})
        fseek(fIn(j), firstSample * bytesPerSample, 'bof');
        dat(j,:) = fread(fIn(j), nSamples, 'int16');
    end
    
    % log data to be returned
    if returnData
        data.v = [data.v; dat * vScale];
        data.chanNames = [data.chanNames, chanNames(ampIdx{i})];
    end
    
    % plot data
    if makePlot
        figure
        for j = 1:length(ampIdx{i})
            subplot(length(ampIdx{i}), 1, j)
            plot(times, dat(j,:) * vScale)
            ylim(yRange);
            xlim([times(1) times(end)]);
            w = ampIdx{i}(j);
            title(['channel ' num2str(w) ', ' rhdInfo.amp(w).custom_name '    impedance: ' num2str(rhdInfo.amp(w).impedance/10^6) ' MOhm'])
        end
    end
    
    % close Intan data files
    arrayfun(@(f) fclose(f), fIn);
    
end

if ~returnData
    data = [];
end