function LoadIntan(datDir, varargin)

% Used to load Intan data files in the "all-channels-in-one-file" format
% Will save select channels in .hsd (version 4) format

% hard-coded parameters
bytesPerSample = 2;

saveChan = {'C-108'};
% ECog = C-108
saveChan = {'S23_E04'};

% default parameter settings
timeRange = [0 Inf];
blockDur = 10;       % size of data chunks (in sec) to be processed at a time
lfpFs = 500;

for i = 1:2:length(varargin)
    switch lower(varargin{i})
        case 'timerange'
            timeRange = varargin{i+1};
    end
end

% create save directory
pathParts = strsplit(datDir, filesep);
pathParts{end} = [pathParts{end} '-filtered'];
saveDir = strjoin(pathParts, filesep);

% load RHD info
rhdFile = [datDir filesep 'info.rhd'];
rhdInfo = ReadRhdInfo(rhdFile);
Fs = rhdInfo.Fs;
nChan = length(rhdInfo.amp);

% create base hsdHeader
headerBase.comment = '';
headerBase.version = 4;
headerBase.sampling_rate = Fs;
headerBase.vScale = 0.000195;
headerBase.bytesPerSample = bytesPerSample;
headerBase.dataOffset = 20480;

% extract "custom name" recorded channels
chanNames = arrayfun(@(x) x.custom_name, rhdInfo.amp, 'UniformOutput', false);

% set up data loading by block
ampFile = [datDir filesep 'amplifier.dat'];
datInfo = dir(ampFile);
datalength = datInfo.bytes / (bytesPerSample * nChan);
blockSize   = round(Fs * blockDur);  
overlapSize = round(Fs * 0.1);   % 100 ms overlap between adjacent blocks to avoid edge effects
t1 = max(0, timeRange(1));
t2 = min(datalength / Fs, timeRange(2));
firstSample = round(t1 * Fs);
nSamples = floor((t2 - t1) * Fs);
finalSample = firstSample + nSamples - 1;
numBlocks = ceil(nSamples / blockSize);
gcp;

if ~exist(saveDir, 'dir')
    mkdir(saveDir);
end

% open files for saving filtered data
r = round(Fs / lfpFs);
header = headerBase;
for i = 1:length(saveChan)
    chanIdx(i) = find(cellfun(@(c) strcmp(saveChan{i}, c), chanNames));
    header.sampling_rate = Fs / r;
    header.num_channels = 1;
    header.channel.chip = rhdInfo.amp(chanIdx(i)).board_stream;
    header.channel.chip_channel = rhdInfo.amp(chanIdx(i)).chip_channel;
    header.channel.impedance = rhdInfo.amp(chanIdx(i)).impedance;
    header.channel.name = lower(rhdInfo.amp(chanIdx(i)).custom_name);
    header.channel.low_cut = rhdInfo.bandwidth(1);
    header.channel.high_cut = min(Fs / (2*r), rhdInfo.bandwidth(2));
    writeHsdHeader_ver4([saveDir filesep chanNames{chanIdx(i)} '.hsdf'], header);
    fOut(i) = fopen([saveDir filesep chanNames{chanIdx(i)} '.hsdf'], 'a');  
end

% load data, filter, save filtered data
f = fopen(ampFile);
for i = 1:numBlocks
    
    % load data from Intan files
    currentSample = firstSample + (i-1) * blockSize;
    preSize = min(currentSample, overlapSize);
    postSize = min(finalSample - currentSample - preSize - blockSize + 1, overlapSize);
    blkSize = blockSize;
    if postSize < 0
        postSize = 0;
        blkSize = finalSample - currentSample - preSize + 1;
    end
    fseek(f, (currentSample - preSize) * bytesPerSample * nChan, 'bof');
    dat = double(fread(f, [nChan preSize+blkSize+postSize], 'int16'));
    dat = dat(chanIdx,:);
        
    % filter and downsample data
    lfpDat = int16(zeros(length(chanIdx), ceil(size(dat, 2) / r)));
    parfor j = 1:length(chanIdx)
        lfpDat(j,:) = int16(decimate(dat(j,:), r));
    end
        
    % write filtered data block to hsdf file for each ECoG/EMG channel
    preSize = floor(preSize/r);
    for j = 1:length(chanIdx)
        fwrite(fOut(j), lfpDat(j, preSize + (1 : min(size(lfpDat,2), floor(blkSize/r)))), 'int16');     
    end 
    
end

fclose(f);
arrayfun(@(f) fclose(f), fOut);    % close hsdf files

