function [spk wire ampl] = DetectSpikes(data, thr, varargin)

% 8/5/2016  M. Farries
% USAGE
% This function assumes that data are from just one tetrode/stereotrode

% REQUIRED INPUTS:
%   data - 2-D array, (# channels) X (# time samples)
%   thr - spike detection threshold--should always be >0 (see below)

% OUTPUTS:

% ** deadTime and overlapTime are in SAMPLE NUMBERS and should be integers
% VARARGs:
%   deadTime - minimum ISI between spikes
%   overlapTime - peaks within overlapTime are assumed to be part of the same spike
%   
%   upsampleRatio - not currently used
%   spkDetectSignal - signal on which spikes are detected
%       SNLE - nonlinear energy
%       abs - absolute value of voltage (best if baseline ~0)
%       pos - unmodified voltage
%       neg - negative voltage (can be used to find negative peaks, best if baseline ~0)

% REQUIRED M-FILES:
%   snle.m

spkDetectSignal = 'SNLE';
s = 1;              % peak "scale" for FindPeaks, in sample numbers
deadTime = 1;       % in sample numbers
overlapTime = 3;    % in sample numbers

for iarg = 1 : 2 : length(varargin)
    
    switch lower(varargin{iarg})
        case 'spkdetectsignal',
            spkDetectSignal    = varargin{iarg + 1};
        case 'deadtime',
            deadTime = varargin{iarg + 1};
        case 'overlaptime',
            overlapTime = varargin{iarg + 1};
        otherwise
            disp('invalid optional argument passed to DetectSpikes')
            return
    end
    
end

numWires = size(data,1);
wireList = 1:numWires;

switch lower(spkDetectSignal)
    case 'snle'
        signal = snle(data, wireList)';
    case 'abs'
        signal = abs(data)';
    case 'neg'
        signal = -data';
    otherwise
        signal = data';
end

%% (1) find signal peaks above threshold

peaks = FindPeaks(signal, s, 1);
[spk wire] = ind2sub(size(peaks), find(peaks));
ampl = signal(peaks);
overThr = ampl > thr(wire);
spk = spk(overThr)';
wire = wire(overThr)';
ampl = ampl(overThr)';
[spk iSort] = sort(spk);
wire = wire(iSort);
ampl = ampl(iSort);

%% (2) remove multiple peaks that might be part of a single spike (i.e., interval < overlapTime)

% identify blocks of ISIs < overlapTime
delIdx = []; % indicies of spikes to be deleted for overlapTime violations
isi = diff(spk);
shortIsiIdx = find(isi < overlapTime);
if ~isempty(shortIsiIdx)
    endLocs = find(diff(shortIsiIdx) > 1); % locations of the ends of sequences of ISIs < overlapTime
    startLocs = endLocs + 1; % locations of the starts of sequences of ISIs < overlapTime
    endLocs = [shortIsiIdx(endLocs) shortIsiIdx(end)] + 1;
    startLocs = [shortIsiIdx(1) shortIsiIdx(startLocs)];
else
    startLocs = [];
end

% for each block of short ISIs, keep the spike with the biggest peak and delete the rest
for i = 1:length(startLocs)
    spkIdx = startLocs(i):endLocs(i);
    amplSamp = ampl(spkIdx);
    spkIdx(amplSamp == max(amplSamp)) = [];
    delIdx = [delIdx spkIdx];
end

% remove spikes marked for death
spk(delIdx) = [];
wire(delIdx) = [];
ampl(delIdx) = [];

%% (3) remove spikes to avoid ISI < deadTime

% identify blocks of ISIs < deadTime
delIdx = []; % indicies of spikes to be deleted for deadTime violations
isi = diff(spk);
shortIsiIdx = find(isi < deadTime);
if ~isempty(shortIsiIdx)
    endLocs = find(diff(shortIsiIdx) > 1); % locations of the ends of sequences of ISIs < deadTime
    startLocs = endLocs + 1; % locations of the starts of sequences of ISIs < deadTime
    endLocs = [shortIsiIdx(endLocs) shortIsiIdx(end)] + 1;
    startLocs = [shortIsiIdx(1) shortIsiIdx(startLocs)];
else
    startLocs = [];
end

% loop through blocks of short ISIs
for k = 1:length(startLocs)
    spkIdx = startLocs(k):endLocs(k);
    amplSamp = ampl(spkIdx);
    % find the spike within the block with maximum amplitude, remove flanking spikes
    maxLoc = find(amplSamp == max(amplSamp));
    tmpDel = maxLoc(1) + [-1 1];
    tmpDel(tmpDel < 1 | tmpDel > length(spkIdx)) = [];
    delIdx = [delIdx spkIdx(tmpDel)];
    spkIdx(tmpDel) = [];
    amplSamp(tmpDel) = [];
    tmpIsi = diff(spk(spkIdx));
    while any(tmpIsi < deadTime)
        % remove from consideration spikes no longer participating in short ISIs
        goodIsis = [0 find(tmpIsi >= deadTime) length(spkIdx)];
        goodSpks = goodIsis(diff(goodIsis) == 1) + 1;
        spkIdx(goodSpks) = [];
        amplSamp(goodSpks) = [];
        tmpIsi = diff(spk(spkIdx));
        % keep the spike with the largest amplitude, remove flanking spikes with ISI violations
        maxLoc = find(amplSamp == max(amplSamp));
        maxLoc = maxLoc(1);
        badIsis = find(tmpIsi < deadTime);
        tmpDel = [];
        if ismember(maxLoc(1), badIsis)
            tmpDel = maxLoc(1) + 1;
        end
        if ismember(maxLoc(1) - 1, badIsis)
            tmpDel = [tmpDel maxLoc(1) - 1];
        end
        delIdx = [delIdx spkIdx(tmpDel)];
        spkIdx(tmpDel) =[];
        amplSamp(tmpDel) = [];
        tmpIsi = diff(spk(spkIdx));
    end
end

% remove spikes marked for death
spk(delIdx) = [];
wire(delIdx) = [];
ampl(delIdx) = [];