function [score splitVal] = Bimodality(dat,varargin)

showPlot = false;
nBins = 100;
quantRange = [0.01 0.99];
smoothFactor = 5;
minSampPerBin = 5;

if length(dat) < minSampPerBin
    score = 0;
    splitVal = nan;
    return
end

for i = 1:2:length(varargin)
    switch lower(varargin{i})
        case 'showplot'
            showPlot = varargin{i+1};
        case 'nbins'
            nBins = varargin{i+1};
        case 'quantrange'
            quantRange = varargin{i+1};
        case 'smoothfactor'
            smoothFactor = varargin{i+1};
        otherwise
            disp('invalid optional argument passed to Bimodality')
    end
end

minX = quantile(dat, quantRange(1));
maxX = quantile(dat, quantRange(2));
dat = dat(dat>=minX & dat <=maxX);
binWidth = range(dat) / nBins;
binEdges = min(dat) - 2*smoothFactor*binWidth : binWidth : max(dat) + 2*smoothFactor*binWidth;
if length(binEdges) < 3
    score = 0;
    splitVal = nan;
    return
end
h = histcounts(dat, binEdges);
% check to see if there are too few samples to assess bimodality
if sum(h) < 2*minSampPerBin
    score = 0;
    splitVal = nan;
    return
end
% if there isn't a bin with at least minSampPerBin, increase bin size
while max(h) < minSampPerBin
    binWidth = 1.5*binWidth;
    binEdges = min(dat) - 2*smoothFactor*binWidth : binWidth : max(dat) + 2*smoothFactor*binWidth;
    h = histcounts(dat, binEdges);
end
x = binEdges(2:end) - binWidth/2;
p = my_conv(h, smoothFactor);
dp = diff(p);

% peaks are extrema with a negative second derivative
peaks = find(dp(1:end-1)>0 & dp(2:end)<0) + 1;
% special case if endpoints are maxima
if dp(1) < 0
    peaks = [1 peaks];
end
if dp(end) > 0
    peaks = [peaks length(p)];
end

% some valleys are extrema with a positive second derivative...
valley = find(dp(1:end-1)<0 & dp(2:end)>0) + 1;
depth = p(valley);
% ...but others are extended regions where p = 0
zeroLocs = find(p == 0);
if ~isempty(zeroLocs)
    endLocs = find(diff(zeroLocs) > 1);
    startLocs = endLocs + 1;
    endLocs = [zeroLocs(endLocs) zeroLocs(end)];
    startLocs = [zeroLocs(1) zeroLocs(startLocs)];
    valley = unique([valley round(mean([startLocs; endLocs], 1))]);
    depth = [depth startLocs-endLocs];
end

% special case if p is constant for one or more consecutive points (but not zero)
flatLocs = find(dp == 0);
if ~isempty(flatLocs)
    endLocs = find(diff(flatLocs) > 1);
    startLocs = endLocs + 1;
    endLocs = [flatLocs(endLocs) flatLocs(end)];
    startLocs = [flatLocs(1) flatLocs(startLocs)];
    for i = 1:length(startLocs)
        if startLocs(i) == 1
            if dp(endLocs(i)+1) < 0
                peaks = [peaks endLocs(i)];
            elseif p(endLocs(i)) ~= 0
                valley = [valley endLocs(i)];
                depth = [depth p(endLocs(i))];
            end
        else
            if dp(startLocs(i)-1) > 0
                peaks = [peaks startLocs(i)];
            elseif p(startLocs(i)) ~= 0
                valley = [valley startLocs(i)];
                depth = [depth p(startLocs(i))];
            end
        end
    end
end

% m3 = skewness(dat,0);
% m4 = kurtosis(dat,0) - 3;
% n = length(dat);
% score2 = (m3^2 + 1)/(m4 + 3*(n-1)^2/((n-2)*(n-3)));

% define height of peak pairs relative to the valley between them
if length(peaks) > 1
    maxHt = 0;
    for i = 1:length(peaks)-1
        for j = i+1:length(peaks)
            valIdx = valley(valley > peaks(i) & valley < peaks(j));
            val = min(p(valIdx));
            pHt = mean(p([peaks(i) peaks(j)])) - val;
            if pHt > maxHt
                maxHt = pHt;
                bestPeaks = [peaks(i) peaks(j)];
            end
        end 
    end
    peaks = bestPeaks; % keep the two highest peaks
    depth = depth(valley>peaks(1) & valley<peaks(2)); % keep valleys between those peaks
    valley = valley(valley>peaks(1) & valley<peaks(2)); % keep valleys between those peaks
    iVal = find(depth == min(depth)); % find and keep the lowest valley
    valley = valley(iVal(1));
    n1 = sum(p(1:valley));
    n2 = sum(p(valley+1:end));
    mu0 = sum(p .* x)/(n1+n2);
    var0 = sum(p .* x.^2)/(n1+n2) - mu0^2;
    mu1 = sum(p(1:valley) .* x(1:valley)) / n1;
    mu2 = sum(p(valley+1:end) .* x(valley+1:end)) / n2;
    var1 = sum(p(1:valley) .* x(1:valley).^2)/n1 - mu1^2;
    var2 = sum(p(valley+1:end) .* x(valley+1:end).^2)/n2 - mu2^2;
    % my bimodality score is the KiloSort score multiplied by relative valley depth
    score = (1 - abs(n1*var1 - n2*var2)/((n1+n2)*var0)) * prod(p(peaks) - p(valley)) / max(p(peaks))^2;
    % score = (1 - abs(n1*var1 - n2*var2)/((n1+n2)*var0)) * (min(p(peaks)) - p(valley)) / max(p(peaks));
    % score = (1 - abs(n1*var1 - n2*var2)/((n1+n2)*var0)) * (min(p(peaks)) - p(valley)) / max(p(peaks)) * (x(peaks(2)) - x(peaks(1)))/std(dat);
    splitVal = x(valley);
else
    score = 0;
    splitVal = nan;
end

% old way
% if length(peaks) > 1
%     [~, iPk] = sort(p(peaks),'descend');
%     peaks = sort(peaks(iPk(1:2))); % keep the two highest peaks
%     depth = depth(valley>peaks(1) & valley<peaks(2)); % keep valleys between those peaks
%     valley = valley(valley>peaks(1) & valley<peaks(2)); % keep valleys between those peaks
%     iVal = find(depth == min(depth)); % find and keep the lowest valley
%     valley = valley(iVal(1));
%     n1 = sum(p(1:valley));
%     n2 = sum(p(valley+1:end));
%     mu0 = sum(p .* x)/(n1+n2);
%     var0 = sum(p .* x.^2)/(n1+n2) - mu0^2;
%     mu1 = sum(p(1:valley) .* x(1:valley)) / n1;
%     mu2 = sum(p(valley+1:end) .* x(valley+1:end)) / n2;
%     var1 = sum(p(1:valley) .* x(1:valley).^2)/n1 - mu1^2;
%     var2 = sum(p(valley+1:end) .* x(valley+1:end).^2)/n2 - mu2^2;
%     % my bimodality score is the KiloSort score multiplied by relative valley depth
%     score = (1 - abs(n1*var1 - n2*var2)/((n1+n2)*var0)) * prod(p(peaks) - p(valley)) / max(p(peaks))^2;
%     % score = (1 - abs(n1*var1 - n2*var2)/((n1+n2)*var0)) * (min(p(peaks)) - p(valley)) / max(p(peaks));
%     % score = (1 - abs(n1*var1 - n2*var2)/((n1+n2)*var0)) * (min(p(peaks)) - p(valley)) / max(p(peaks)) * (x(peaks(2)) - x(peaks(1)))/std(dat);
%     splitVal = x(valley);
% else
%     score = 0;
%     splitVal = nan;
% end

if showPlot
    figure
    plot(x,h,'r')
    hold on
    plot(x,p,'k')  
    scatter(x(peaks(1)), p(peaks(1)), 'g', 'filled')
    if length(peaks) > 1
        scatter(x(peaks(2)), p(peaks(2)), 'g', 'filled')
        plot([mu1 mu1], ylim, 'g')
        plot([mu2 mu2], ylim, 'g')
        plot([splitVal splitVal], ylim, 'b')
    end
    hold off
    xlim([x(1) x(end)])
    title(['bimodality score: ' num2str(score)]) 
end

    