function spkDat = LoadNexWaves(nexFile, trodeFile, varargin)

wL = 1.6;               % wavelength, in ms
pk = 0.7;               % peak location, in ms
wP = 0.35;              % wavePad, in ms
alignWin = [-0.7 0.7];  % alignWin, in ms (not used much at this juncture)
upsample = 2;           % upsampling ratio
batchSize = 32768;
verbose = true;
spkDat.sincLength = 12;
wireSd = 500;           % ballpark guess for wire SD, needed for some spikeworks functions (can add option to measure it in this function!)

for i = 1:2:length(varargin)
    switch lower(varargin{i})
        case 'peakloc'
            pk = varargin{i+1};             % time of spk peak within spk wave, in ms
        case 'wavelength'
            wL = varargin{i+1};             % length of spk waves, in ms
        case 'wavepad'
            wP = varargin{i+1};             % data "pad" flanking spk wave that can be used for wave realignment, in ms
        case 'verbose'
            verbose = varargin{i+1};        % determines whether load progress is reported during execution
        case 'batchsize'
            batchSize = varargin{i+1};      % size of data batches for loading, in samples
        case 'upsample'
            upsample = varargin{i+1};       % factor by which data are upsampled (with sinc interpolation)
        case 'vscale'
            vScale = varargin{i+1};         % scale factor converting raw data into mV
        otherwise
            disp('Invalid optional argument passed to ReloadWaves')
    end
end

% get the hsd header
hsdInfo    = dir(trodeFile);
hsdHeader  = getHSDHeader( trodeFile );
Fs         = hsdHeader.main.sampling_rate;
dataOffset = hsdHeader.dataOffset;
nChan   = hsdHeader.main.num_channels;   % total number of wires in the .hsd file
if hsdHeader.main.version < 4
    bytes_per_sample = 2;
    voltRange = [-10 10];
    numBits = 8 * bytes_per_sample;
    spkDat.vScale = 1000 * (range(voltRange) / 2^numBits) / hsdHeader.channel(1).gain;
% else
%     bytes_per_sample = hsdHeader.x; % for future version of hsdHeader
%     vScale = hsdHeader.x; % for future version of hsdHeader
end
datalength = floor((hsdInfo.bytes - dataOffset) / (bytes_per_sample * nChan)); 
wireList = 1 : nChan;
spkDat.highCut = hsdHeader.channel(1).high_cut;

% convert wave parameters from ms to sample numbers, log info in spkDat
wL = round(wL * Fs * upsample / 1000);
pk = round(pk * Fs * upsample / 1000);
wP = round(wP * Fs * upsample / 1000);
spkDat.alignWin = round(alignWin * Fs * upsample / 1000);
spkDat.peakLoc = pk;
spkDat.upsample = upsample;
spkDat.wavePad = wP;
spkDat.Fs = Fs;
spkDat.wireSd = ones(1, nChan) * wireSd;

% load nex data
[nCells cellName type] = nex_info(nexFile);
spkDat.unitNames = cell(1, nCells);
spkDat.ts = [];
spkDat.id = [];
for i = 1:nCells
    spkDat.unitNames{i} = deblank(cellName(i,:));
    spkDat.unitNames{i} = spkDat.unitNames{i}(end);
    [~, ts] = nex_ts(nexFile, cellName(i,:));
    spkDat.ts = [spkDat.ts ts];
    spkDat.id = [spkDat.id ones(1, length(ts)) * i];
end
[spkDat.ts, iSort] = sort(spkDat.ts);
spkDat.ts = round(spkDat.ts * Fs);
spkDat.id = spkDat.id(iSort);

% eliminate duplicate spikes
[~, idx] = unique(spkDat.ts);
delIdx = setdiff(1:length(spkDat.ts), idx);
spkDat.ts(delIdx) = [];
spkDat.id(delIdx) = [];
spkDat.ts0 = spkDat.ts;
spkDat.tsOrig = spkDat.ts;

% load spike waveforms
tic;
ts = spkDat.ts;
spkDat.ts0 = ts;
startSample = min(ts) - wP - pk;
endSample = max(ts) + wP + wL - pk;
startSample = max([1 startSample]);
endSample = min([endSample datalength]);
numSamples = endSample - startSample + 1;
overlap = 4*wL + 2*wP;
numBatches = ceil((numSamples - 2*wL)/(batchSize - overlap));
if verbose
    fprintf('\nLoading spike waveforms, batch ');
    msg = '';
end
tsLocal = [];
spkDat.waves = zeros(wL + 2*wP, nChan, length(spkDat.ts));
for n = 1:numBatches
    
    if verbose
        fprintf(repmat('\b', 1, numel(msg)));
        msg = sprintf('%d of %d, time elapsed: %d seconds', n, numBatches, round(toc));
        fprintf(msg);
    end
    
    % load data from hsd file
    dat = readHSD(trodeFile, nChan, dataOffset, Fs, [], 'usesamplelimits', [startSample, batchSize]);
    if upsample > 1
        tmpDat = zeros(size(dat, 1), size(dat, 2) * upsample);
        for w = 1:nChan
            tmpDat(w, :) = sincInterp(dat(w, :), Fs, spkDat.highCut, upsample*Fs, 'sinclength', spkDat.sincLength);
        end
        dat = tmpDat;
    end
    
    % extract spike waveforms
    tsLocal = setdiff(ts(ts >= startSample + wP + pk & ts < startSample + batchSize - wP - wL + pk), tsLocal);
    [~, loc] = ismember(tsLocal, ts);
    spk = (tsLocal - startSample + 1) * upsample;
    if ~isempty(spk)        
        idx = bsxfun(@plus, spk - pk - wP, (1:wL + 2*wP)');
        wv = reshape(dat(:,idx), nChan, wL + 2*wP, length(spk)); % #chan X #pts X #spikes
        spkDat.waves(:,:,loc) = permute(wv, [2 1 3]); % #chan X #pts X #spikes
    end
    
    startSample = startSample + batchSize - overlap;
    
end

if verbose
    fprintf('...load complete\n ');
end
