function PlotClusters_AM(spkDat, clustList, varargin)

% convert waveform data from int16 to double-precision floating point
spkDat.waves = double(spkDat.waves);

maxScale = Inf;
refWave = [];
sameScale = false;
showMean = true;
convertToVolt = true;
quantiles = [];
minIsi = 1; % in milliseconds
nSampTarget = 500;
rateBin = 10;
timeRange = [0 Inf];
peakValHeatmap = true;

for i = 1:2:length(varargin)
    switch lower(varargin{i})
        case 'showmean'
            showMean = varargin{i+1};
        case 'converttovolt'
            convertToVolt = varargin{i+1};
        case 'maxscale'
            maxScale = varargin{i+1};
        case 'compare'
            refWave = varargin{i+1}; % should have dimensions (nSamples, nChan)
        case 'samescale'
            sameScale = varargin{i+1};
        case 'quantiles'
            quantiles = varargin{i+1};
        case 'minisi'
            minIsi = varargin{i+1};
        case 'nsamp'
            nSampTarget = varargin{i+1};
        case 'timerange'
            timeRange = varargin{i+1}; % recording time range to plot, in minutes
        case 'peakValHeatmap'
            peakValHeatmap = varargin{i+1}; % recording time range to plot, in minutes
        otherwise
            disp('invalid optional argument passed to PlotClusters');
    end
end

clustList = intersect(clustList, setdiff(unique(spkDat.id), 0));
nClust = length(clustList);
[wL, nChan, ~] = size(spkDat.waves);
wP = spkDat.wavePad;
wL = wL - 2*wP;
pk = spkDat.peakLoc;
alignWin = spkDat.alignWin;
vScale = spkDat.vScale;
Fs = spkDat.Fs;
wvFs = Fs * spkDat.upsample;

timeRange = timeRange * 60; % convert timeRange from minutes to seconds
tR = timeRange * spkDat.Fs;
loc = find(spkDat.ts >= tR(1) & spkDat.ts <= tR(2));
spkDat.ts = spkDat.ts(loc);
spkDat.id = spkDat.id(loc);
spkDat.waves = spkDat.waves(:,:,loc);

% assemble plot data, calculate y Range
yRange = zeros(nClust, 2);
if convertToVolt
    refWave = vScale * refWave;
end
refMin = [];
refMax = [];
if ~isempty(refWave)
    refMin = min(min(refWave));
    refMax = max(max(refWave));
end
avgWave = zeros(wL, nChan, nClust);
samp = cell(1, nClust);
loc = cell(1, nClust);
q = zeros(wL, nChan, length(quantiles), nClust);
isiViol = zeros(1, nClust);
pts = (1:wL) + wP;
for i = 1:nClust
    
    % compute average waveform
    n = clustList(i);
    loc{i} = find(spkDat.id == n);
    nSpk = length(loc{i});
    if nSpk > 0
        avgWave(:,:,i) = mean(spkDat.waves(pts,:,loc{i}), 3);
    end
    if convertToVolt
        avgWave(:,:,i) = vScale * avgWave(:,:,i);
    end
    
    % compute isi violation
    if nSpk > 1
        spkTimes = 1000 * spkDat.ts(loc{i}) / Fs; % spike times in ms
        isi = diff(spkTimes);
        isiViol(i) = 100 * length(find(isi < minIsi)) / length(isi);
    else
        isiViol(i) = nan;
    end
    
    % compute quantiles
    for k = 1:length(quantiles)
        for w = 1:nChan
            q(:,w,k,i) = arrayfun(@(j) quantile(spkDat.waves(j,w,loc{i}), quantiles(k)), pts);
        end
    end
    if convertToVolt
        q = vScale * q;
    end
    
    % select sample of waveforms to plot
    wv = zeros(wL, nChan, 1);
    nSamp = min([nSampTarget nSpk]);
    samp{i} = randsample(nSpk, nSamp);
    if nSpk > 1
        samp{i} = loc{i}(samp{i});
        wv = spkDat.waves(pts,:,samp{i});
    end
    if convertToVolt
        wv = vScale * wv;
    end
    tmpMin = min([min(min(wv, [], 3)) refMin]);
    tmpMax = max([max(max(wv, [], 3)) refMax]);
    
    % compute yRange
    avgMin = min(min(avgWave(:,:,i)));
    avgMax = max(max(avgWave(:,:,i)));
    avgRange = avgMax - avgMin;
    yRange(i,1) = max([tmpMin, avgMin - avgRange * maxScale]);
    yRange(i,2) = min([tmpMax, avgMax + avgRange * maxScale]);
    if nSpk == 0
        yRange(i,:) = [-0.1 0.1];
    end
    
end

if sameScale
    yRange(:,1) = min(yRange(:,1));
    yRange(:,2) = max(yRange(:,2));
end

% make figure panel
scrsz = get(0,'ScreenSize');
scrWidth = scrsz(3);
scrHeight = scrsz(4);
vertSpace = 2;
vertSpace2 = 35;
horizSpace = 45;
plotWidth = 300*.65;
plotHeight = 280*.65;
histHeight = 180*.6;
panelHeight = vertSpace2 + nChan * (vertSpace + plotHeight) + 4 * (vertSpace2 + histHeight) + vertSpace2 - vertSpace;
panelWidth = horizSpace + nClust * (horizSpace + plotWidth);
fig = figure('Position', [0.05*scrWidth 0.95*scrHeight - panelHeight panelWidth panelHeight]); %[left bottom width height];
spV = vertSpace/panelHeight;
spV2 = vertSpace2/panelHeight;
spH = horizSpace/panelWidth;
wd = plotWidth/panelWidth;
ht = plotHeight/panelHeight;
hh = histHeight/panelHeight;

% plot data for each cluster
t = double(1000 * ((1:wL) - pk)) / wvFs;
for i = 1:nClust
    
    wv = zeros(wL, nChan, 1);
    if ~isempty(samp{i})
        wv = spkDat.waves(pts,:,samp{i});
    end
    if convertToVolt
        wv = vScale * wv;
    end
    
    % plot wire waveforms
    for w = 1:nChan
        subplot('Position', [spH+(i-1)*(wd+spH) spV+(nChan-w)*(ht+spV)+4*(hh+spV2)+spV2-spV  wd ht]) %[left bottom width height]
        set(gca, 'Box', 'on')
        if w==1
            if isfield(spkDat, 'unitNames')
                str = spkDat.unitNames{i};
                uName = [str(1:5) ' ' str(6:9) ' ' str(10:11) '-' str(12:13) ' ' str(14:end)];
                title([uName ': ' num2str(length(loc{i})) ' spikes'])
            else
                title(['unit ' num2str(clustList(i)) ': ' num2str(length(loc{i})) ' spikes'])
            end
        end
        hold on
        plot(t, squeeze(wv(:,w,:)), 'LineWidth', 1, 'color', 'k')
        if ~isempty(refWave)
            plot(t, refWave(:,w), 'LineWidth', 3, 'color', 'c')
        end
        if showMean
            plot(t, avgWave(:,w,i), 'LineWidth', 3, 'color', 'r')
        end
        for k = 1:length(quantiles)
            plot(t, q(:,w,k,i), 'LineWidth', 1.5, 'color', 'g')
        end
        xlim([t(1) t(end)]);
        yR = yRange(i,:) + [-0.01 0.01] * range(yRange(i,:));
        plot([0 0], yR, 'LineWidth', 1, 'color', 'b');
        ylim(yR);
        yt = get(gca, 'YTick');
        dy = ceil(diff(yt(1:2))/0.1)*0.1;
        yt = ceil(yR(1)/dy)*dy:dy:floor(yR(2)/dy)*dy;
        set(gca, 'YTick', yt)
        ylabel('mV')
        if w==nChan
            xlabel('time relative to spike (ms)')
        else
            set(gca, 'XTickLabel', [])
        end
    end
    
    % plot ISI histogram
    subplot('Position', [spH+(i-1)*(wd+spH) spV2+3*(hh+spV2)  wd hh]) %[left bottom width height]
    spkTimes = [];
    if length(loc{i} > 2)
        spkTimes = spkDat.ts(loc{i}) / Fs;
        isi = 1000 * diff(spkTimes);
        iqr = quantile(isi, 0.75) - quantile(isi, 0.25);
        binEdges = linspace(0, 3*iqr, 100);
        h = histogram(isi, binEdges);
        h.LineWidth = 0.01;
        h.EdgeAlpha = 0;
        h.FaceAlpha = 1;
        h.FaceColor = 'k';
        ax = gca;
        xlim([min(binEdges) max(binEdges)])
        isiStr = [num2str(isiViol(i), '%1.2f') '%'];
        xr = xlim;
        xPos = 0.85*range(xr) + xr(1);
        yr = ylim;
        yPos = 0.93*range(yr) + yr(1);
        text(xPos, yPos, isiStr, 'FontSize', 12, 'BackgroundColor', 'w', 'Color', 'r', 'FontWeight', 'Bold');
    end
    ax.XLabel.String = 'ISI (ms)';
    ax.YTick = [];
    
    % plot firing rate and spike amplitude
    [~, maxChan] = max(abs(avgWave(pk,:,i)));
    t1 = 0;
    if isfinite(timeRange(1))
        t1 = timeRange(1);
    end
    if isfinite(timeRange(2))
        t2 = timeRange(2);
    elseif length(spkDat.ts) > 0
        t2 = ceil(spkDat.ts(end)/(rateBin*Fs))*rateBin;
    else
        t2 = 0;
    end
    if (t2 - t1) / rateBin < 20
        rateBin = (t2 - t1) / 20;
    end
    binEdges = t1:rateBin:t2;
    [rate, ~, bin] = histcounts(spkTimes, binEdges);
    rate = rate / rateBin;
    binCenters = (binEdges(2:end) - diff(binEdges)/2)/60;
    ampl = zeros(size(binCenters));
    if ~isempty(loc{i})
        pol = sign(avgWave(pk,maxChan,i));
        for j = 1:length(ampl)
            ampl(j) = mean(spkDat.waves(pk+wP, maxChan, loc{i}(bin == j))) * vScale * pol;
        end
    end
    subplot('Position', [spH+(i-1)*(wd+spH) spV2+2*(hh+spV2)  wd hh]) %[left bottom width height]
    scatter(binCenters, rate, 'filled')
    xlabel('time in session (minutes)')
    ylabel('firing rate (Hz)')
    xlim([binEdges(1)/60, binEdges(end)/60])
    maxRate = round(max(rate)/5)*5 + 5;
    ylim([0 maxRate])
    set(gca, 'Box', 'on')
    subplot('Position', [spH+(i-1)*(wd+spH) spV2+(hh+spV2)  wd hh]) %[left bottom width height]
    scatter(binCenters, ampl, 'filled')
    maxAmpl = round(max(ampl)/0.1)*0.1 + 0.1;
    ylim([0 maxAmpl])
    yt = get(gca, 'YTick');
    dy = ceil(diff(yt(1:2))/0.1)*0.1;
    yt = 0:dy:floor(maxAmpl/dy)*dy;
    set(gca, 'YTick', yt)
    xlabel('time in session (minutes)')
    ylabel('spike amplitude (mV)')
    xlim([binEdges(1)/60, binEdges(end)/60])
    set(gca, 'Box', 'on')
    
    subplot('Position', [spH+(i-1)*(wd+spH) spV2  wd hh]) %[left bottom width height]
    
    if peakValHeatmap
        
        ampl =[];
        if ~isempty(loc{i})
            pol = sign(avgWave(pk,maxChan,i));
            for j = 1:length(rate)
                ampl(j,:) = hist(spkDat.peakVal(loc{i}(bin == j)),0:5:2000);
                medAmp(j) = nanmedian(spkDat.peakVal(loc{i}(bin == j)));
            end
        end
        pcolor(binCenters,0:5:2000,ampl');ylim([0 200]);hold on
        plot(binCenters,medAmp,'linestyle','--','color','w')
        for iWire = maxChan%1:nChan
            line([0 max(binCenters)],[spkDat.spkThr(iWire) spkDat.spkThr(iWire)],'linestyle','--','color','r','linewidth',2)
        end
        for task=1:length(spkDat.taskEnd)-1
            line([spkDat.taskEnd(task)/60 spkDat.taskEnd(task)/60],[0 200],'linestyle','--','color','y','linewidth',2)
        end
        shading interp;
        ylabel('spike amplitude (mV)')
        xlabel('time in session (minutes)')
    else
        [n xout] = hist(spkDat.peakVal(loc{i}),0:5:2000);
        bar(xout,n,1,'FaceColor','b','EdgeColor','b');xlim([0 300])
        hold on;
        for iWire = maxChan%1:nChan
            line([spkDat.spkThr(iWire) spkDat.spkThr(iWire)],[0 max(n)],'linestyle','--','color','r','linewidth',2)
        end
        set(gca, 'YTick', yt)
        ylabel('Peak Distribution')
        xlabel('spike amplitude (mV)')
    end

    set(gca, 'Box', 'on')
end
