function spkDat = ReloadWaves2(spkDat, trodeFile, varargin)

[wL, nChan, nSpk] = size(spkDat.waves);
pk = double(spkDat.peakLoc);
wP = double(spkDat.wavePad);
wL = wL - 2*wP;
upsample = spkDat.upsample;
batchSize = 32768;
verbose = true;
whiten = false;
alignWin = spkDat.alignWin;
% convert wave parameters to ms
wL = 1000 * wL / (spkDat.Fs * upsample);
pk = 1000 * pk / (spkDat.Fs * upsample);
wP = 1000 * wP / (spkDat.Fs * upsample);
alignWin = 1000 * alignWin / (spkDat.Fs * upsample);

for i = 1:2:length(varargin)
    switch lower(varargin{i})
        case 'peakloc'
            pk = varargin{i+1};             % time of spk peak within spk wave, in ms
        case 'wavelength'
            wL = varargin{i+1};             % length of spk waves, in ms
        case 'wavepad'
            wP = varargin{i+1};             % data "pad" flanking spk wave that can be used for wave realignment, in ms
        case 'verbose'
            verbose = varargin{i+1};        % determines whether load progress is reported during execution
        case 'batchsize'
            batchSize = varargin{i+1};      % size of data batches for loading, in samples
        case 'upsample'
            upsample = varargin{i+1};       % factor by which data are upsampled (with sinc interpolation)
        case 'whiten'
            whiten = varargin{i+1};         % determines whether to apply channel-whitening filter to data
        otherwise
            disp('Invalid optional argument passed to ReloadWaves')
    end
end

if ~iscell(trodeFile)
    trodeFile = {trodeFile};
end

% get the hsd header
hsdHeader = getHsdHeader_ver4(trodeFile{1});
Fs = hsdHeader.sampling_rate;
bytesPerSample = hsdHeader.bytesPerSample;
dataOffset = hsdHeader.dataOffset;
nChan = hsdHeader.num_channels; 

% convert wave parameters from ms to sample numbers, log info in spkDat
wL = int32(wL * Fs * upsample / 1000);
pk = int32(pk * Fs * upsample / 1000);
wP = int32(wP * Fs * upsample / 1000);
spkDat.alignWin = round(alignWin * Fs * upsample / 1000);
spkDat.peakLoc = double(pk);
spkDat.upsample = upsample;
spkDat.wavePad = double(wP);

if isfield(spkDat, 'taskStart')
    nTask = length(spkDat.taskStart);
    nFile = length(trodeFile);
    if nTask ~= nFile
        disp('WARNING--number of trode files does not match number of spkDat tasks');
    end
    nFile = min(nFile, nTask);
end

% load spike waveforms
tic;
spkDat.ts0 = spkDat.ts;
if verbose
    fprintf('\nLoading spike waveforms, ');
    msg = '';
end
if whiten
    spkDat.waves = zeros(wL + 2*wP, nChan, nSpk);
else
%    spkDat.waves = int16(zeros(wL + 2*wP, nChan, nSpk));
end

for k = 1:nFile
    
    hsdInfo = dir(trodeFile{k});
    datalength = floor((hsdInfo.bytes - dataOffset) / (bytesPerSample * nChan));
    
    ts = spkDat.ts(spkDat.ts >= spkDat.taskStart(k)*Fs & spkDat.ts <= spkDat.taskEnd(k)*Fs);
    ts = int32(ts - spkDat.taskStart(k)*Fs);
    startSample = min(ts) - wP - pk;
    endSample = max(ts) + wP + wL - pk;
    startSample = max([0 startSample]);
    endSample = min([endSample datalength]);
    numSamples = endSample - startSample + 1;
    overlap = 4*wL + 2*wP;
    numBatches = ceil((numSamples - 2*wL)/(batchSize - overlap));
    
    tsLocal = [];
    for n = 1:numBatches
    
        if verbose
            fprintf(repmat('\b', 1, numel(msg)));
            msg = sprintf('file %d of %d, batch %d of %d, time elapsed: %d seconds', k, nFile, n, numBatches, round(toc));
            fprintf(msg);
        end
        
        if n == 5000 | n == 8835
    
        % load data from hsd file
        dat = 
        dat = int16(readHSD(trodeFile{k}, nChan, dataOffset, Fs, [], 'usesamplelimits', [startSample, batchSize]));
        if upsample > 1
            tmpDat = int16(zeros(size(dat, 1), size(dat, 2) * upsample));
            
            for w = 1:nChan
                tmpDat(w, :) = int16(sincInterp(double(dat(w, :)), Fs, spkDat.highCut, upsample*Fs, 'sinclength', spkDat.sincLength));
            end
            dat = tmpDat;
        end
        
        else
            startSample = startSample + batchSize - overlap;
            continue
        end
           
        % apply channel-whitening filter
        if whiten
            dat = spkDat.W * double(dat);
        end
    
        % extract spike waveforms
        tsLocal = setdiff(ts(ts >= startSample + wP + pk & ts < startSample + batchSize - wP - wL + pk), tsLocal);
        [~, loc] = ismember(tsLocal + spkDat.taskStart(k) * Fs, spkDat.ts);
        spk = (tsLocal - startSample) * upsample;
        if any(tsLocal > 160*60*Fs)
            disp('whoa');
        end
        if ~isempty(spk)        
            idx = bsxfun(@plus, spk - pk - wP, (1:wL + 2*wP)');
            wv = reshape(dat(:,idx), nChan, wL + 2*wP, length(spk)); % #chan X #pts X #spikes
            spkDat.waves(:,:,loc) = permute(wv, [2 1 3]); % #chan X #pts X #spikes
        end
    
        startSample = startSample + batchSize - overlap;
    
    end

end

if verbose
    fprintf('...load complete\n');
end
