function spkDat = RemoveShortIsi(spkDat, varargin)

% Removes duplicate waveforms (typically created by waveform realigment) identified by short ISI.
% This function can fix ISI violation problems (not necessarily a good thing)
% The waveform best matching its cluster mean is kept, other ISI violators are removed.
% win and chan are used to select what parts of the waveform are compared to the template

minIsi = 0.2; % in ms
[wL, nChan, ~] = size(spkDat.waves);
chanList = 1:nChan;
clustList = setdiff(unique(spkDat.id), 0);
pk = spkDat.peakLoc;
win = 1000 * [1 - pk wL - pk] / spkDat.Fs;
verbose = true;

for i = 1:2:length(varargin)
    switch lower(varargin{i})
        case 'minisi'
            minIsi = varargin{i+1};
        case 'clust'
            clustList = varargin{i+1};
        case 'chan'
            chanList = varargin{i+1};
        case 'win'
            win = varargin{i+1};
    end
end

clustList = intersect(clustList, setdiff(unique(spkDat.id), 0));
nClust = length(clustList);
win = round(spkDat.Fs * win / 1000); % convert ms to sample numbers
minIsi = round(spkDat.Fs * minIsi / 1000); % convert ms to sample numbers
maxWin = [1 - pk, wL - pk];
win(1) = max([win(1) maxWin(1)]);
win(2) = min([win(2) maxWin(2)]);
nPts = range(win) + 1;
pts = (1:nPts) + win(1) + pk - 1;
nChanIn = length(chanList);
sd = [];
for i = 1:nChanIn
    sd = [sd; ones(nPts,1) * spkDat.wireSd(chanList(i))];
end

locMain = find(ismember(spkDat.id, clustList));
nSpk = length(locMain);
[~,id] = ismember(spkDat.id(locMain), clustList);
ts = spkDat.ts(locMain);
wv = spkDat.waves(pts, chanList, locMain);
wv = reshape(wv, nPts*nChanIn, nSpk);
[ts, iSort] = sort(ts);
locMain = locMain(iSort);
id = id(iSort);
wv = wv(:, iSort);

% compute templates
template = zeros(nPts*nChanIn, nClust);
for i = 1:nClust
    loc = find(id == i);
    template(:,i) = mean(wv(:,loc), 2);
end

% measure error for every spike
err = zeros(1, nSpk);
for k = 1:nSpk
    err(k) = mean(((template(:,id(k)) - wv(:,k)) ./ sd).^2);
end

% identify blocks of ISIs < minIsi
delIdx = []; % indicies of spikes to be deleted for ISI violations
isi = diff(ts);
shortIsiIdx = find(isi < minIsi);
if ~isempty(shortIsiIdx)
    endLocs = find(diff(shortIsiIdx) > 1); % locations of the ends of sequences of ISIs < minIsi
    startLocs = endLocs + 1; % locations of the starts of sequences of ISIs < minIsi
    endLocs = [shortIsiIdx(endLocs) shortIsiIdx(end)] + 1;
    startLocs = [shortIsiIdx(1) shortIsiIdx(startLocs)];
else
    startLocs = [];
end

% loop through blocks of short ISIs
for k = 1:length(startLocs)
    idx = startLocs(k):endLocs(k);
    errSamp = err(idx);
    % find the spike within the block with minimum error, remove flanking spikes
    minLoc = find(errSamp == min(errSamp));
    tmpDel = minLoc(1) + [-1 1];
    tmpDel(tmpDel < 1 | tmpDel > length(idx)) = [];
    delIdx = [delIdx idx(tmpDel)];
    idx(tmpDel) = [];
    errSamp(tmpDel) = [];
    tmpIsi = diff(ts(idx));
    while any(tmpIsi < minIsi)
        % remove from consideration spikes no longer participating in short ISIs
        goodIsis = [0 find(tmpIsi >= minIsi) length(idx)];
        goodSpks = goodIsis(diff(goodIsis) == 1) + 1;
        idx(goodSpks) = [];
        errSamp(goodSpks) = [];
        tmpIsi = diff(ts(idx));
        % keep the spike with the smallest error, remove flanking spikes with ISI violations
        minLoc = find(errSamp == min(errSamp));
        minLoc = minLoc(1);
        badIsis = find(tmpIsi < minIsi);
        tmpDel = [];
        if ismember(minLoc(1), badIsis)
            tmpDel = minLoc(1) + 1;
        end
        if ismember(minLoc(1) - 1, badIsis)
            tmpDel = [tmpDel minLoc(1) - 1];
        end
        delIdx = [delIdx idx(tmpDel)];
        idx(tmpDel) =[];
        errSamp(tmpDel) = [];
        tmpIsi = diff(ts(idx));
    end
end

if verbose
    for i = 1:nClust
        nDel = length(find(id(delIdx) == i));
        fprintf('%d spikes removed from cluster %d\n', nDel, clustList(i));
    end
end

% move bad spikes into the unsorted pile
delIdx = locMain(delIdx);
spkDat.id(delIdx) = 0;
spkDat.ts(delIdx) = spkDat.tsOrig(delIdx);


spkDat = spkDat;