function traj = CalculateMotion(aChip1, aChip2, param)


% unpack inital conditions
x = param.x;   % position vector (of board center relative to box)
Oz = param.Oz; % vector specifying orientation of board top
Oy = param.Oy; % vector specifying orientation of board front
v = param.v;   % linear velocity vector
w = param.w;   % angular velocity vector

% unpack parameters
r1board = param.r1board; % position of chip 1, relative to board center
r2board = param.r2board; % position of chip 2, relative to board center
dt = param.dt;           % sampling interval for accelerometer data
g = param.g;             % gravity vector (should be [0; 0; 980] with good calibration)

nPts = size(aChip1, 2);
traj.t = (0:nPts-1)*dt + param.t;
traj.x = zeros(3,nPts);
traj.Oz = zeros(3,nPts);
traj.Oy = zeros(3,nPts);
traj.v = zeros(3,nPts);
traj.w = zeros(3,nPts);
traj.a = zeros(3,nPts);
traj.wdot = zeros(3,nPts);
for i = 1:nPts
    
    % log position and velocity
    traj.x(:,i) = x;
    traj.Oz(:,i) = Oz;
    traj.Oy(:,i) = Oy;
    traj.v(:,i) = v;
    traj.w(:,i) = w;
    
    % compute matrix that will rotate board frame into box frame
    Rz = RotMatrix(Oz, [0;0;1]);
    Ry = RotMatrix(Rz*Oy, [0;-1;0]);
    R = Ry * Rz;
    
    % convert position vectors of accelerometer chips to box frame
    r1 = R * r1board;
    r2 = R * r2board;
    
    % convert chip acceleration to box acceleration, subtract g
    a1 = R * aChip1(:,i);
    a2 = R * aChip2(:,i);
    a1 = a1 - g;
    a2 = a2 - g;
    
    % define linear acceleration as mean of a1, a2
    a = mean([a1 a2],2);
    
    % subtract linear acceleration and rotation-driven acceleration from a1, a2
    a1 = a1 - a + cross(w, cross(w, r1));
    a2 = a2 - a + cross(w, cross(w, r2));
    
    % compute angular acceleration from remainder of a1, a2
    A = [0,      r1(3), -r1(2);
        -r1(3),  0,      r1(1);
         r1(2), -r1(1),  0;
         0,      r2(3), -r2(2);
        -r2(3),  0,      r2(1);
         r2(2), -r2(1),  0];
    B = [a1; a2];
    wdot = linsolve(A,B);
    
    % log acceleration
    traj.a(:,i) = a;
    traj.wdot(:,i) = wdot;
    
    % update position and orientation vectors for next timestep
    x = x + dt * v;
    Oz = Oz + dt * cross(w, Oz);
    Oy = Oy + dt * cross(w, Oy);

    % update linear and angular velocity vectors for next timestep
    v = v + dt * a;
    w = w + dt * wdot;
    
end